// camera.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "camera.h"
#include "film.h"
#include "mc.h"
// Camera Method Implementations
Camera::~Camera() {
	delete film;
}
Camera::Camera(const Transform &world2cam, Float hither, Float yon,
		Float sopen, Float sclose, Film *f) {
	WorldToCamera = world2cam;
	CameraToWorld = WorldToCamera.GetInverse();
	ClipHither = hither;
	ClipYon = yon;
	invClipHither = 1.f / ClipHither;
	ShutterOpen = sopen;
	ShutterClose = sclose;
	film = f;
}
ProjectiveCamera::ProjectiveCamera(const Transform &w2c,
		const Transform &proj, const Extent2D &Screen,
		Float hither, Float yon, Float sopen,
		Float sclose, Float lensr,
		Float focald, Film *f)
	: Camera(w2c, hither, yon, sopen, sclose, f) {
	// Initialize depth of field parameters
	LensRadius = lensr;
	FocalDistance = focald;
	// Compute projective camera transformations
	CameraToScreen = proj;
	WorldToScreen = CameraToScreen * WorldToCamera;
	// Compute projective camera screen transformations
	ScreenToRaster = Scale(film->xResolution-1.f, film->yResolution-1.f, 1.f) *
		Scale(1.f / (Screen.x1 - Screen.x0),
		      1.f / (Screen.y0 - Screen.y1), 1.f) *
		Translate(Vector(-Screen.x0, -Screen.y1, 0.f));
	RasterToScreen = ScreenToRaster.GetInverse();
	RasterToCamera = CameraToScreen.GetInverse() * RasterToScreen;
}
Float ProjectiveCamera::ScreenDepth(const Point &Pworld) const {
	return WorldToScreen(Pworld).z;
}
