// camera.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef CAMERA_H
#define CAMERA_H
#include "lrt.h"
#include "color.h"
#include "sampling.h"
#include "geometry.h"
#include "transform.h"
// Camera Declarations
class Camera {
public:
	Camera(const Transform &world2cam,
		Float hither, Float yon, Float sopen, Float sclose,
		Film *film);
	virtual ~Camera();
	// Camera Interface Declarations
	virtual void GenerateRay(const Sample *sample,
		Ray &ray) const = 0;
	virtual Float ScreenDepth(const Point &Pworld) const = 0;
	// Camera Public Data
	Film *film;
	Transform WorldToCamera, CameraToWorld;
protected:
	// Camera Options
	Float ClipHither, ClipYon, invClipHither;
	Float ShutterOpen, ShutterClose;
};
class ProjectiveCamera : public Camera {
public:
	// ProjectiveCamera Method Declarations
	ProjectiveCamera(const Transform &world2cam, const Transform &proj,
		const Extent2D &Screen, Float hither, Float yon, Float sopen,
		Float sclose, Float lensr, Float focald, Film *film);
	virtual Float ScreenDepth(const Point &Pworld) const;
protected:
	// ProjectiveCamera Options
	Transform CameraToScreen, WorldToScreen, RasterToCamera;
	Transform ScreenToRaster, RasterToScreen;
	Float LensRadius, FocalDistance;
};
#endif // CAMERA_H
