// film.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef FILM_H
#define FILM_H
#include "lrt.h"
#include "color.h"
#include "geometry.h"
#include "transform.h"
// Film Declarations
class Film {
public:
	// Film Interface
	Film(int xres, int yres, const Extent2D &Crop);
	virtual ~Film();
	inline bool UpdatePixel(int x, int y, const Spectrum &L,
			Float alpha, Float depth, Float weight) {
		if (x < xPixelStart || x >= xPixelStart + xPixelWidth ||
			y < yPixelStart || y >= yPixelStart + yPixelWidth)
			return false;
		int offset = (y - yPixelStart) * xPixelWidth +
			(x - xPixelStart);
		Pixel *pixelp = pixels + offset;
		pixelp->L += L * weight;
		pixelp->alpha += alpha * weight;
		pixelp->depth = min(pixelp->depth, depth);
		pixelp->weightSum += weight;
		return true;
	}
	void WriteCoefficients(const string &filename) const;
	void WriteDisplayImage(const DisplayInfo &dinfo) const;
	// Film Public Data
	int xResolution, yResolution;
private:
	// Film Private Data
	int xPixelStart, yPixelStart, xPixelWidth, yPixelWidth;
	struct Pixel {
		Pixel() { alpha = 0.; depth = INFINITY; weightSum = 0.; }
		Spectrum L;
		Float alpha, depth, weightSum;
	};
	Pixel *pixels;
};
struct DisplayInfo {
	DisplayInfo() {
		// DisplayInfo Constructor Implementation
		toneMap = NULL;
		maxDisplayY = 100.f;
		displayAdaptationY = 50.f;
		rWeight[0] =  3.240479f;
		rWeight[1] = -1.537150f;
		rWeight[2] = -0.498535f;
		gWeight[0] = -0.969256f;
		gWeight[1] =  1.875991f;
		gWeight[2] =  0.041556f;
		bWeight[0] =  0.055648f;
		bWeight[1] = -0.204043f;
		bWeight[2] =  1.057311f;
		clampToGamut = true;
		invGamma = 1.f / 2.2f;
		integerFormat = true;
		maxDisplayValue = 255;
		ditherAmount = 0.5f;
		filename = "lrt.tiff";
	}
	// DisplayInfo Data
	ToneMap *toneMap;
	Float maxDisplayY, displayAdaptationY;
	Float rWeight[3], gWeight[3], bWeight[3];
	bool clampToGamut;
	Float gain, invGamma;
	bool integerFormat;
	int maxDisplayValue;
	Float ditherAmount;
	string filename;
};
#endif // FILM_H
