// light.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "light.h"
#include "scene.h"
// Light Method Definitions
Light::~Light() {
}
Spectrum Light::L(const Point &, const Point &, VisibilityTester *) const {
	return Spectrum(0.);
}
bool VisibilityTester::Unoccluded(const Scene *scene) const {
  // Update shadow ray statistics
  static StatsCounter nShadowRays("Lights",
				  "Number of shadow rays traced");
  ++nShadowRays;
  return !scene->IntersectP(r);
}
Spectrum VisibilityTester::Transmittance(const Scene *scene) const {
  return scene->Transmittance(r);
}
Spectrum MultiAreaLight::Le(const RayDifferential &r) const {
  return parent->Le(r) * scale;
}
Spectrum MultiAreaLight::L(const Point &x, const Vector &w) const {
  return parent->L(x, w) * scale;
}
Spectrum MultiAreaLight::B(const Point &x) const {
  return parent->B(x) * scale;
}
Spectrum MultiAreaLight::L(const Point &Ps, const Point &Pl,
			   VisibilityTester *visibility) const {
  return parent->L(Ps, Pl, visibility) * scale;
}
Spectrum MultiAreaLight::dE(const Point &P, const Normal &N,
			    Vector *w, VisibilityTester *vis) const {
  return parent->dE(P, N, w, vis) * scale;
}
Spectrum Light::Le(const RayDifferential &) const {
  return Spectrum(0.);
}
Spectrum Light::Sample_L(const Point &P, const Normal &N, Float u1, Float u2,
			 Vector *wo, Float *weight, bool *deltaLight,
			 VisibilityTester *visibility) const {
  return Sample_L(P, u1, u2, wo, weight, deltaLight, visibility);
}
Spectrum MultiAreaLight::Sample_L(const Point &P, Float u1, Float u2, Vector *wo, Float *weight,
				  bool *deltaLight, VisibilityTester *vis) const {
  return parent->Sample_L(P, u1, u2, wo, weight, deltaLight, vis) * scale;
}
Spectrum MultiAreaLight::Sample_L(const Point &P, const Normal &N, Float u1, Float u2,
				  Vector *wo, Float *weight, bool *deltaLight,
				  VisibilityTester *vis) const {
  return parent->Sample_L(P, N, u1, u2, wo, weight, deltaLight, vis) * scale;
}
Spectrum MultiAreaLight::Sample_L(const Scene *scene, Float u1, Float u2,
		Float u3, Float u4, Ray *ray, Float *weight,
				  bool *deltaLight) const {
  return parent->Sample_L(scene, u1, u2, u3, u4, ray, weight, deltaLight);
}
Float MultiAreaLight::Weight(const Point &P, const Normal &N,
			     const Vector &w) const {
  return parent->Weight(P, N, w);
}
Float MultiAreaLight::Weight(const Point &P, const Vector &w) const {
  return parent->Weight(P, w);
}
