// materials.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "materials.h"
#include "color.h"
#include "reflection.h"
#include "texture.h"
#include "shapes.h"
// Material Method Definitions
Material::~Material() {
	delete displace;
}
void Material::Bump(const DifferentialGeometry &dgg,
		DifferentialGeometry *dgs) const {
	if (!displace)
		return;
	// Compute offset positions and evaluate displacement texture
	DifferentialGeometry dgdx, dgdy;
	dgs->Shift(1, 0, &dgdx);
	dgs->Shift(0, 1, &dgdy);
	Point P = dgs->P + Vector(dgs->Nn) * displace->Evaluate(*dgs);
	Point Px = dgdx.P + Vector(dgdx.Nn) * displace->Evaluate(dgdx);
	Point Py = dgdy.P + Vector(dgdy.Nn) * displace->Evaluate(dgdy);
	// Compute bump-mapped differential geometry
	Float dx = sqrtf(dgs->dudx*dgs->dudx + dgs->dvdx*dgs->dvdx);
	Float dy = sqrtf(dgs->dudy*dgs->dudy + dgs->dvdy*dgs->dvdy);
	dgs->UpdateBasis((P-Px) / dx, (P-Py) / dy);
}
Texture<Spectrum> *Material::MakeSpecTex(const ParamSet &pGeom, const ParamSet &pShader,
		const string &name, const Spectrum &def) {
	int type, narray;
	const Spectrum *s = pGeom.FindSpectrum(name, &type, &narray);
	if (!s)   s = pShader.FindSpectrum(name, &type, &narray);
	if (!s) return new ConstantTexture<Spectrum>(def);
	Assert(narray == 1); // XXX for now

	if (type & PARAM_TYPE_UNIFORM)
		return new ConstantTexture<Spectrum>(*s);
	else if (type & PARAM_TYPE_VARYING)
		return new BilerpTexture<Spectrum>(new IdentityMapping2D,
			s[0], s[1], s[2], s[3]);
	else {
		if (pGeom.nVertex != 0)
			return new VertexTexture<Spectrum>(s, pGeom.nVertex);
		else {
			Warning("Vertex texture for \"%s\" not supported "
				"for this object", name.c_str());
			return NULL;
		}
	}
}
Texture<Float> *Material::MakeFloatTex(const ParamSet &pGeom, const ParamSet &pShader,
		const string &name, Float def) {
	int type, narray;
	const Float *s = pGeom.FindFloat(name, &type, &narray);
	if (!s)
		s = pShader.FindFloat(name, &type, &narray);
	if (!s) return new ConstantTexture<Float>(def);
	Assert(narray == 1); // XXX for now
	if (type & PARAM_TYPE_UNIFORM)
		return new ConstantTexture<Float>(*s);
	else if (type & PARAM_TYPE_VARYING)
		return new BilerpTexture<Float>(new IdentityMapping2D,
			s[0], s[1], s[2], s[3]);
	else {
		if (pGeom.nVertex != 0)
			return new VertexTexture<Float>(s, pGeom.nVertex);
		else {
			Warning("Vertex texture for \"%s\" not supported "
				"for this object", name.c_str());
			return NULL;
		}
	}
}
