// materials.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef MATERIALS_H
#define MATERIALS_H
#include "lrt.h"
#include "primitives.h"
#include "texture.h"
#include "color.h"
#include "reflection.h"
// Material Class Declarations
class Material : public ReferenceCounted<Material> {
public:
	// Material Interface
	virtual BSDF *GetBSDF(const Intersection *isect) const = 0;
	Material(Texture<Float> *disp) {
		displace = disp;
		istranslucent=false;
		isphoton=false;;
	}
	virtual ~Material();
	void Bump(const DifferentialGeometry &dgg, DifferentialGeometry *dgShading) const;
	static Texture<Spectrum> *MakeSpecTex(const ParamSet &pGeom, const ParamSet &pShader,
			const string &name, const Spectrum &def);
	static Texture<Float> *MakeFloatTex(const ParamSet &pGeom, const ParamSet &pShader,
		const string &name, Float def);
	
	bool istranslucent;
	bool isphoton;
private:
	// Material Private Data
	Texture<Float> *displace;
};
// Material creation macros
#define SURF_TEX_S(var, def) \
	Texture<Spectrum> *(var) = Material::MakeSpecTex(geomParams, surfaceParams, \
		#var, def)
#define SURF_TEX_F(var, def) \
	Texture<Float> *(var) = Material::MakeFloatTex(geomParams, surfaceParams, \
		#var, def)
#define DISP_TEX_S(var, def) \
	Texture<Spectrum> *(var) = Material::MakeSpecTex(geomParams, displaceParams, \
		#var, def)
#define DISP_TEX_F(var, def) \
	Texture<Float> *(var) = Material::MakeFloatTex(geomParams, displaceParams, \
		#var, def)
#define DISP_FLOAT_PARAM(name, default) \
	geomParams.FindOneFloat(name, \
		displaceParams.FindOneFloat(name, default))
#endif // MATERIALS_H
