// mc.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef MC_H
#define MC_H
// MC Utility Declarations
extern Vector UniformSampleHemisphere(Float u1, Float u2);
extern Vector UniformSampleSphere(Float u1, Float u2);
extern Vector UniformSampleCone(Float u1, Float u2, Float thetamax);
extern Vector UniformSampleCone(Float u1, Float u2, Float thetamax,
	const Vector &x, const Vector &y, const Vector &z);
extern Float UniformConeWeight(Float thetamax);
inline Vector CosineSampleHemisphere(Float u1, Float u2) {
	Vector ret;
	ConcentricSampleDisk(u1, u2, &ret.x, &ret.y);
	ret.z = sqrtf(max(0.f, 1.f - ret.x*ret.x - ret.y*ret.y));
	return ret;
}
extern void UniformSampleDisk(Float u1, Float u2, Float *x, Float *y);
extern void ConcentricSampleDisk(Float u1, Float u2, Float *x, Float *y);
// MC Class Declarations
class Constant2DSampler {
public:
	// Constant2DSampler Method Declarations
	Constant2DSampler(Float *f, int nx, int ny);
	~Constant2DSampler();
	void Sample(Float u1, Float u2, Float *x, Float *y, Float *weight) const;
private:
	// Constant2DSampler Private Data
	Float *f, *fx;
	int nx, ny;
	Float *xcdf;
	Float **ycdfs;
	Float xc, *ycs;
};
#endif
