// paramset.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "paramset.h"
// ParamSet Methods
ParamSet &ParamSet::operator=(const ParamSet &p2) {
	if (&p2 != this) {
		clear();
		u_int i;
		for (i = 0; i < p2.ints.size(); ++i)
			ints.push_back(p2.ints[i]->Clone(nVertex));
		for (i = 0; i < p2.floats.size(); ++i)
			floats.push_back(p2.floats[i]->Clone(nVertex));
		for (i = 0; i < p2.points.size(); ++i)
			points.push_back(p2.points[i]->Clone(nVertex));
		for (i = 0; i < p2.vectors.size(); ++i)
			vectors.push_back(p2.vectors[i]->Clone(nVertex));
		for (i = 0; i < p2.normals.size(); ++i)
			normals.push_back(p2.normals[i]->Clone(nVertex));
		for (i = 0; i < p2.spectra.size(); ++i)
			spectra.push_back(p2.spectra[i]->Clone(nVertex));
		for (i = 0; i < p2.strings.size(); ++i)
			strings.push_back(p2.strings[i]->Clone(nVertex));
	}
	return *this;
}
void ParamSet::AddFloat(const string &name, const Float *data,
		int type, int narray) {
	type |= PARAM_TYPE_FLOAT;
	floats.push_back(new ParamSetItem<Float>(name, data, type,
		narray, nVertex));
}
#define ADD_PARAM_TYPE(T, vec) \
	(vec).push_back(new ParamSetItem<T>(name, (T *)data, type, \
		narray, nVertex))
void ParamSet::AddInt(const string &name, const int *data, int type,
		int narray) {
	type |= PARAM_TYPE_INT;
	ADD_PARAM_TYPE(int, ints);
}
void ParamSet::AddPoint(const string &name, const Point *data, int type, int narray) {
	type |= PARAM_TYPE_POINT;
	ADD_PARAM_TYPE(Point, points);
}
void ParamSet::AddVector(const string &name, const Vector *data, int type, int narray) {
	type |= PARAM_TYPE_VECTOR;
	ADD_PARAM_TYPE(Vector, vectors);
}
void ParamSet::AddNormal(const string &name, const Normal *data, int type, int narray) {
	type |= PARAM_TYPE_NORMAL;
	ADD_PARAM_TYPE(Normal, normals);
}
void ParamSet::addSpectrum(const string &name, const Spectrum *data, int type, int narray) {
	type |= PARAM_TYPE_COLOR;
	ADD_PARAM_TYPE(Spectrum, spectra);
}
void ParamSet::AddString(const string &name, const string *data, int narray) {
	int type = PARAM_TYPE_STRING | PARAM_TYPE_UNIFORM;
	ADD_PARAM_TYPE(string, strings);
}
Float ParamSet::FindOneFloat(const string &name, Float d) const {
	for (u_int i = 0; i < floats.size(); ++i)
		if (floats[i]->name == name &&
			floats[i]->type & PARAM_TYPE_UNIFORM &&
			floats[i]->arraySize == 1) {
			floats[i]->lookedUp = true;
			return *(floats[i]->data);
		}
	return d;
}
const Float *ParamSet::FindFloat(const string &name, int *type,
		int *nArray) const {
	for (u_int i = 0; i < floats.size(); ++i)
		if (floats[i]->name == name) {
			*nArray = floats[i]->arraySize;
			*type = floats[i]->type;
			floats[i]->lookedUp = true;
			return floats[i]->data;
		}
	return NULL;
}
#define LOOKUP_PTR(vec) \
	for (u_int i = 0; i < (vec).size(); ++i) \
		if ((vec)[i]->name == name) { \
			*nArray = (vec)[i]->arraySize; \
			*type = (vec)[i]->type; \
			(vec)[i]->lookedUp = true; \
			return (vec)[i]->data; \
		} \
	return NULL
const int *ParamSet::findInt(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(ints);
}
#define LOOKUP_ONE(vec) \
	for (u_int i = 0; i < (vec).size(); ++i) { \
		if ((vec)[i]->name == name && \
			(vec)[i]->type & PARAM_TYPE_UNIFORM && \
			(vec)[i]->arraySize == 1) { \
			(vec)[i]->lookedUp = true; \
			return *((vec)[i]->data); \
}		} \
	return d
int ParamSet::FindOneInt(const string &name, int d) const {
	LOOKUP_ONE(ints);
}
const Point *ParamSet::FindPoint(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(points);
}
Point ParamSet::FindOnePoint(const string &name, const Point &d) const {
	LOOKUP_ONE(points);
}
const Vector *ParamSet::FindVector(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(vectors);
}
Vector ParamSet::FindOneVector(const string &name, const Vector &d) const {
	LOOKUP_ONE(vectors);
}
const Normal *ParamSet::FindNormal(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(normals);
}
Normal ParamSet::FindOneNormal(const string &name, const Normal &d) const {
	LOOKUP_ONE(normals);
}
const Spectrum *ParamSet::FindSpectrum(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(spectra);
}
Spectrum ParamSet::FindOneSpectrum(const string &name, const Spectrum &d) const {
	LOOKUP_ONE(spectra);
}
const string *ParamSet::FindString(const string &name, int *type, int *nArray) const {
	LOOKUP_PTR(strings);
}
string ParamSet::FindOneString(const string &name, const string &d) const {
	LOOKUP_ONE(strings);
}
int ParamSet::TypeToNum(int type) {
	if (type & PARAM_TYPE_UNIFORM) return 1;
	else if (type & PARAM_TYPE_VARYING) return 4;
	else if (type & PARAM_TYPE_VERTEX)  return nVertex;
	else {
		Assert(1 == 0);
		return 1;
	}
}
void ParamSet::ReportUnused() const {
#define CHECK_UNUSED(v) \
	for (i = 0; i < (v).size(); ++i) \
		if (v[i]->name[0] != '_' && !(v)[i]->lookedUp) \
			Warning("Parameter \"%s\" not used", \
				(v)[i]->name.c_str())
	u_int i;
	CHECK_UNUSED(ints);
	CHECK_UNUSED(floats);
	CHECK_UNUSED(points);
	CHECK_UNUSED(vectors);
	CHECK_UNUSED(normals);
	CHECK_UNUSED(spectra);
	CHECK_UNUSED(strings);
}
ParamSet::~ParamSet() {
	clear();
}
void ParamSet::clear() {
	u_int i;
#define DEL_PARAMS(name) \
	for (i = 0; i < (name).size(); ++i) \
		delete (name)[i]; \
	(name).erase((name).begin(), (name).end())

	DEL_PARAMS(ints);
	DEL_PARAMS(floats);
	DEL_PARAMS(points);
	DEL_PARAMS(vectors);
	DEL_PARAMS(normals);
	DEL_PARAMS(spectra);
	DEL_PARAMS(strings);
#undef DEL_PARAMS
}
void ParamSet::init(int n, const char **tokens,
		void **params, int nv) {
	nVertex = nv;
	clear();
	// Initialize ParamSet data values
	for (int i = 0; i < n; ++i) {
		int type, narray;
		string name;
		if (lookupType(tokens[i], &type, &narray, name)) {
			// Process successful token lookup for ParamSet
			void *data = params[i];
			if (type & PARAM_TYPE_INT) {
				// parser doesn't handle ints, so convert from floats here....
				int nAlloc = max(1, narray);
				if (type & PARAM_TYPE_VARYING) nAlloc *= 4;
				if (type & PARAM_TYPE_VERTEX) nAlloc *= nVertex;
				int *idata = new int[nAlloc];
				for (int i = 0; i < nAlloc; ++i)
					idata[i] = int(((Float *)data)[i]);
				ints.push_back(new ParamSetItem<int>(name, idata, type, narray, nVertex));
			}
			else if (type & PARAM_TYPE_FLOAT)
				ADD_PARAM_TYPE(Float, floats);
			else if (type & PARAM_TYPE_POINT)
				ADD_PARAM_TYPE(Point, points);
			else if (type & PARAM_TYPE_VECTOR)
				ADD_PARAM_TYPE(Vector, vectors);
			else if (type & PARAM_TYPE_NORMAL)
				ADD_PARAM_TYPE(Normal, normals);
			else if (type & PARAM_TYPE_COLOR)
				ADD_PARAM_TYPE(Spectrum, spectra);
			else if (type & PARAM_TYPE_STRING) {
				string stmp(*((const char **)data));
				ParamSetItem<string> *psi =
					new ParamSetItem<string>(name, &stmp, type,
						narray, nVertex);
				strings.push_back(psi);
			}
		}
		else
			Warning("Type of parameter \"%s\" is unknown",
				tokens[i]);
	}
}
