// primitives.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef PRIMITIVES_H
#define PRIMITIVES_H
#include "lrt.h"
#include "geometry.h"
#include "shapes.h"
#include "transform.h"
#include "color.h"
#include "materials.h"
// Primitive Declarations
class Primitive : public ReferenceCounted<Primitive> {
public:
	// Primitive Interface
	virtual BBox WorldBound() const = 0;
	virtual bool CanIntersect() const;
	virtual bool Intersect(const Ray &r, Intersection *s) const = 0;
	virtual bool IntersectP(const Ray &r) const = 0;
	virtual void Refine(vector<Reference<Primitive> > &refined) const;
	virtual ~Primitive();
};
class GeometricPrimitive: public Primitive {
public:
	// GeometricPrimitive Public Interface
	GeometricPrimitive(const Reference<Shape> &s, const Reference<Material> &m,
		AreaLight *a, Texture<Normal> *Ns, Texture<Vector> *Ss);
	~GeometricPrimitive();
	bool CanIntersect() const;
	void Refine(vector<Reference<Primitive> > &refined) const;
	virtual BBox WorldBound() const;
	virtual bool Intersect( const Ray &r, Intersection *isect ) const;
	virtual bool IntersectP( const Ray &r ) const;
	void GeometricPrimitive::Bump(const DifferentialGeometry &dg,
		DifferentialGeometry *dgShading) const;
	// GeometricPrimitive Data
	Reference<Shape> shape;
	Reference<Material> material;
	AreaLight *areaLight;
	Texture<Normal> *Ns;
	Texture<Vector> *Ss;
};
struct Intersection {
	Intersection() { primitive = NULL; }
	// Intersection Method Declarations
	BSDF *GetBSDF(const RayDifferential &ray) const;
	Spectrum Le(const Vector &wo) const;
	// Intersection Data Members
	DifferentialGeometry dgGeom;
	mutable DifferentialGeometry dgShading;
	const GeometricPrimitive *primitive;
};
// PrimitiveList Declarations
class PrimitiveList : public Primitive {
public:
	// PrimitiveList Public Interface
	PrimitiveList(const vector<Reference<Primitive> > &p);
	int NumPrims() const { return (int) prims.size(); }
	BBox WorldBound() const { return bounds; }
	bool CanIntersect() const;
	bool Intersect(const Ray &r, Intersection *i) const;
	bool IntersectP(const Ray &r) const;
	void Refine(vector<Reference<Primitive> > &refined) const;
protected:
	// PrimitiveList Protected Data
	vector<Reference<Primitive> > prims;
	BBox bounds;
};
#endif // PRIMITIVES_H
