// scene.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef SCENE_H
#define SCENE_H
#include "lrt.h"
#include "primitives.h"
#include "transport.h"


// Scene Declarations
class Scene {
public:
	// Scene Method Declarations
	Scene(Camera *c, SurfaceIntegrator *in, VolumeIntegrator *vi, Sampler *s,
		Primitive *accel, const vector<Light *> &lts,
		const vector<VolumeRegion *> &vrs, DisplayInfo *di);
	~Scene();
	const BBox &Scene::WorldBound() const {
		return bound;
	}
	void Render();
	Spectrum L(const RayDifferential &ray, const Sample *sample,
		Float *alpha = NULL) const;
	Spectrum Transmittance(const Ray &ray) const;
	bool Intersect(const Ray &ray, Intersection *isect) const {
		return prims->Intersect(ray, isect);
	}
	bool IntersectP(const Ray &ray) const {
		return prims->IntersectP(ray);
	}
	// Scene Data
	vector<Light *> lights;
	Primitive *prims;
	Camera *camera;
	DisplayInfo *displayInfo;
	Sampler *sampler;
	SurfaceIntegrator *surfaceIntegrator;
	VolumeIntegrator *volumeIntegrator;
	vector<VolumeRegion *> volumeRegions;
	BBox bound;
	Reference<Material> skymat;
	Texture<Spectrum> *Ksky;
};
#endif // SCENE_H
