// volume.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "volume.h"
// Volume Scattering Definitions
Float PhaseIsotropic(const Vector &w, const Vector &wp) {
	return 1.f / (4. * M_PI);
}
Float PhaseRayleigh(const Vector &w, const Vector &wp) {
	Float costheta = Dot(w, wp);
	return  3.f/(16.f*M_PI) * (1 + costheta * costheta);
}
Float PhaseMieHazy(const Vector &w, const Vector &wp) {
	Float costheta = Dot(w, wp);
	return 9.f/(4.f*M_PI) * powf(1.f + costheta*costheta, 8.f);
}
Float PhaseMieMurky(const Vector &w, const Vector &wp) {
	Float costheta = Dot(w, wp);
	return 50.f/(4.f*M_PI) * powf(1.f + costheta*costheta, 32.f);
}
Float PhaseHG(const Vector &w, const Vector &wp, Float g) {
	Float costheta = Dot(w, wp);
	return 1.f / (4.f * M_PI) * (1.f - g*g) /
		powf(1.f + g*g - 2.f * g * costheta, 1.5f);
}
Float PhaseSchlick(const Vector &w, const Vector &wp, Float g) {
	Float gcostheta = g * Dot(w, wp);
	return 1.f / (4. * M_PI) * (1 - g*g) /
		((1 - gcostheta) * (1 - gcostheta));
}
Spectrum VolumeRegion::sigma_t(const Point &P, const Vector &w) const {
	return sigma_a(P, w) + sigma_s(P, w);
}
Spectrum DensityRegion::tau(const Ray &r) const {
	Float t0, t1;
	if (!Intersect(r, &t0, &t1)) return 0.;
	Spectrum t(0.);
#define NUM 1
	for (int i = 0; i < NUM; ++i) {
		Float tt = Lerp(((Float)i + RandomFloat())/NUM, t0, t1);
		Point P = r(tt);
		t += sigma_a(P, r.D) + sigma_s(P, r.D);
	}
	return t * Distance(r(t0), r(t1)) / NUM;
}
