// volume.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef VOLUME_H
#define VOLUME_H 1
#include "lrt.h"
#include "color.h"
#include "geometry.h"
#include "paramset.h"
#include "transform.h"
#include "transport.h"
// Volume Scattering Declarations
extern Float PhaseIsotropic(const Vector &w, const Vector &wp);
extern Float PhaseRayleigh(const Vector &w, const Vector &wp);
extern Float PhaseMieHazy(const Vector &w, const Vector &wp);
extern Float PhaseMieMurky(const Vector &w, const Vector &wp);
extern Float PhaseHG(const Vector &w, const Vector &wp, Float g);
extern Float PhaseSchlick(const Vector &w, const Vector &wp, Float g);
class VolumeRegion {
public:
	// VolumeRegion Methods
	virtual ~VolumeRegion() { }
	virtual BBox WorldBound() const = 0;
	virtual bool Intersect(const Ray &ray, Float *t0, Float *t1) const = 0;
	virtual Spectrum sigma_a(const Point &, const Vector &) const = 0;
	virtual Spectrum sigma_s(const Point &, const Vector &) const = 0;
	virtual Spectrum Le(const Point &, const Vector &) const = 0;
	virtual Float phase(const Point &, const Vector &,
		const Vector &) const = 0;
	virtual Spectrum sigma_t(const Point &, const Vector &) const = 0;
	virtual Spectrum tau(const Ray &ray) const = 0;
};
class DensityRegion : public VolumeRegion {
public:
	// DensityRegion Methods
	DensityRegion::DensityRegion(const Spectrum &sa, const Spectrum &ss, Float gg,
		 	const Spectrum &emit, const Transform &v2w) {
		WorldToVolume = v2w.GetInverse();
		sig_a = sa;
		sig_s = ss;
		g = gg;
		le = emit;
	}
	virtual Float density(const Point &Pobj) const = 0;
	Spectrum sigma_a(const Point &p, const Vector &) const {
		return density(WorldToVolume(p)) * sig_a;
	}
	Spectrum sigma_s(const Point &p, const Vector &) const {
		return density(WorldToVolume(p)) * sig_s;
	}
	Spectrum sigma_t(const Point &p, const Vector &) const {
		return density(WorldToVolume(p)) * (sig_a + sig_s);
	}
	Spectrum Le(const Point &p, const Vector &) const {
		return density(WorldToVolume(p)) * le;
	}
	Float phase(const Point &p, const Vector &wi, const Vector &wo) const {
		return PhaseHG(wi, wo, g);
	}
	Spectrum tau(const Ray &r) const;
protected:
	// DensityRegion Protected Data
	Transform WorldToVolume;
	Spectrum sig_a, sig_s, le;
	Float g;
};
class VolumeIntegrator : public Integrator {
public:
	// VolumeIntegrator Methods
	virtual Spectrum Transmittance(const Scene *, const Ray &ray, const Sample *sample,
		Float *alpha) const = 0;
};
#endif // VOLUME_H
