// lrt.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef LRT_H
#define LRT_H

extern int debugflag; // MBGP
extern bool isCreateBlob;
extern int snowparam;
#define ADDSKY


// Global Include Files
#ifdef WIN32
#include <malloc.h> // for _alloca
#endif
#include <sys/time.h>
#include <algorithm>
using std::min;
using std::max;
using std::swap;
using std::sort;
#ifdef __linux__
#include <fpu_control.h>
#endif
#ifdef WIN32
#include <float.h>
#endif
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
using std::ostream;
#include <string>
using std::string;
#include <vector>
using std::vector;
//#ifndef __GNUG__
//#include <pair.h>
//#endif // !__GNUG__
using std::pair;
using std::make_pair;
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <assert.h>
// Platform-specific definitions
#ifdef WIN32
#define memalign(a,b) malloc(b)
#endif
#ifdef sgi
#define for if (0) ; else for
#endif
#ifdef WIN32
#pragma warning( disable: 4190 )
// extern "C" nonsense when returning a template
#endif
#ifdef WIN32
#define isnan _isnan
#define isinf(f) (!_finite((f)))
#endif
// Global Type Declarations
typedef double StatsCounterType;
typedef float Float;
typedef unsigned char u_char;
typedef unsigned short u_short;
typedef unsigned int u_int;
typedef unsigned long u_long;
// Global Forward Declarations
class Scene;
class Primitive;
class GeometricPrimitive;
struct Intersection;
struct Matrix4x4;
class ParamSet;
template <class T> struct ParamSetItem;
class Shape;
struct RefinedShape;
class Vector;
class Point;
class Normal;
class Ray;
class RayDifferential;
struct Extent2D;
class BBox;
class Transform;
struct DifferentialGeometry;
class PrimitiveList;
class Spectrum;
class Camera;
class ProjectiveCamera;
class Film;
class Sampler;
struct Sample;
#define BC_GRID_SIZE 40
typedef vector<int> SampleGrid[BC_GRID_SIZE][BC_GRID_SIZE];
#define GRID(v) (int((v) * BC_GRID_SIZE))
class Filter;
struct DisplayInfo;
class ToneMap;
class BxDF;
class BRDF;
class BTDF;
class Fresnel;
class FresnelConductor;
class FresnelDielectric;
class FresnelNoOp;
class SpecularReflection;
class SpecularTransmission;
class Lambertian;
class OrenNayar;
class Microfacet;
class MicrofacetDistribution;
class BSDF;
class Material;
template <class T> class Texture;
template <class T> class MixTexture;
class TextureMapping2D;
class IdentityMapping2D;
class SphericalMapping2D;
class CylindricalMapping2D;
class PlanarMapping2D;
class TextureMapping3D;
class IdentityMapping3D;
template <class T> class BilerpTexture;
template <class T> class VertexTexture;
class UVTexture;
extern Float Noise(Float x, Float y = .5f, Float z = .5f);
extern Float Noise(const Point &P);
extern Float FBm(const Point &P, const Vector &dPdx, const Vector &dPdy,
	Float omega, int octaves);
class Light;
struct VisibilityTester;
class AreaLight;
class VolumeRegion;
class Constant2DSampler;
class SurfaceIntegrator;
class Integrator;
class VolumeIntegrator;
// Global Constants
#ifdef WIN32
#define alloca _alloca
#endif
#ifndef M_PI
#define M_PI           3.14159265358979323846f
#endif
#define INV_255 .00392156862745098039f
#define INV_PI  0.31830988618379067154f
#ifndef INFINITY
#define INFINITY FLT_MAX
#endif
#define LRT_VERSION 1.0
#define RAY_EPSILON 1e-1f
#define COLOR_SAMPLES 3
// Global Function Declarations
// Setup printf format
#ifdef __GNUG__
#define PRINTF_FORMAT __attribute__ \
	((__format__ (__printf__, 1, 2)))
#else
#define PRINTF_FORMAT
#endif // __GNUG__
extern void Info(const char *, ...) PRINTF_FORMAT;
extern void Warning(const char *, ...) PRINTF_FORMAT;
extern void Error(const char *, ...) PRINTF_FORMAT;
extern void Severe(const char *, ...) PRINTF_FORMAT;
extern void StatsPrint(FILE *dest);
extern void StatsCleanup();
void *AllocL1CacheAligned(size_t size);
void *AllocL2CacheAligned(size_t size);
void FreeCacheAligned(void *);
bool SolveLinearSystem2x2(const Float A[2][2], const Float B[2],
	Float x[2]);
extern unsigned long genrand_int32(void);
extern Float genrand_real1(void);
extern Float genrand_real2(void);
extern Transform Translate(const Vector &delta);
extern Transform Scale(Float x, Float y, Float z);
extern Transform RotateX(Float angle);
extern Transform RotateY(Float angle);
extern Transform RotateZ(Float angle);
extern Transform Rotate(Float angle, const Vector &axis);
extern Transform LookAt(const Point &pos, const Point &look, const Vector &up);
extern Transform Orthographic(Float znear, Float zfar);
extern Transform Perspective(Float fov, Float znear, Float zfar);
extern Spectrum FrDiel(Float cosi, Float cost, const Spectrum &etai, const Spectrum &etat);
extern Spectrum FrCond(Float cosi, const Spectrum &n, const Spectrum &k);
extern Spectrum FresnelApproxEta(const Spectrum &intensity);
extern Spectrum FresnelApproxK(const Spectrum &intensity);
extern void ComputeStep1dCDF(Float *f, int nValues, Float *c, Float *cdf);
extern Float SampleStep1d(Float *f, Float *cdf, Float c,
	int nSteps, Float u, Float *weight);
extern Float *Compute2dXCDF(Float *f, int nx, int ny, Float *xc);
extern Float **Compute2dYCDFs(Float *f, int nx, int ny, Float **ycs);
extern void SampleStep2d(Float *f, Float *xcdf, Float **ycdfs,
	int nx, int ny,	Float u1, Float u2, Float *x, Float *y);
extern void ComputeLinear1dCDF(Float *values, int nValues, Float dx, Float *cdf);
extern Float SampleLinear1dCDF(Float *values, Float *cdf, int nValues, Float dx, Float u);
extern Vector RejectionSampleHemisphere();
extern Float UniformHemisphereWeight();
void ConcentricSampleDisk(Float u1, Float u2, Float *dx, Float *dy);
Spectrum UniformSampleAllLights(const Scene *scene, const Point &P,
	const Normal &N, const Vector &wo, BSDF *bsdf,
	const Sample *sample, int lightSampleOffset,
	int bsdfSampleOffset);
Spectrum UniformSampleOneLight(const Scene *scene, const Point &P,
	const Normal &N, const Vector &wo, BSDF *bsdf,
	const Sample *sample, int lightSampleOffset, int bsdfSampleOffset);
Spectrum WeightedSampleOneLight(const Scene *scene, const Point &P,
	const Normal &N, const Vector &wo, BSDF *bsdf,
	const Sample *sample, int lightSampleOffset, int bsdfSampleOffset,
	Float *&avgY, Float *&avgYsample, Float *&cdf, Float &overallAvgY);
Spectrum EstimateDirect(const Scene *scene, const Light *light, const Point &P,
	const Normal &N, const Vector &wo, BSDF *bsdf,
	const Sample *sample, int lightSamp, int bsdfSamp, int sampleNum);
extern Spectrum *TIFFRead(const string &name, int *xSize, int *ySize);
extern void TIFFWrite8Bit(const string &name, Float *pixels, Float *alpha,
	int XRes, int YRes, int nChannels, int totXRes = 0, int totYRes = 0);
extern void TIFFWriteFloat(const string &name, Float *pixels, Float *alpha,
	int XRes, int YRes, int nChannels, int totXRes = 0, int totYRes = 0);
extern void InitAPI();
extern void CleanupAPI();
extern bool ParseFile(const char *filename);
// Global Classes
struct ProgressReporter {
	// ProgressReporter Methods
	ProgressReporter(int t, const string &ti, FILE *o = stderr,
			int wid = 65)
		: width(wid - ti.size()), frequency(t / width), total(t) {
		count = 0;
		nPlusses = 0;
		gettimeofday(&start, NULL);
		title = ti;
		out = o;
	}
	void Update(int num = 1) const {
		count -= num;
		while (count <= 0) {
			count += frequency;
			// Update progress plus signs
			fprintf(out, "\r%s: [", title.c_str());
			++nPlusses;
			for (int i = 0; i < nPlusses; ++i)
				fprintf(out, "+");
			for (int i = 0; i < width - nPlusses; ++i)
				fprintf(out, " ");
			fprintf(out, "]");
			// Update elapsed time and estimated time to completion
			struct timeval now;
			gettimeofday(&now, NULL);
			Float percentDone = (Float)nPlusses / (Float)width;
			Float seconds = now.tv_sec - start.tv_sec +
				(now.tv_usec - start.tv_usec) / 1e6f;
			Float estRemaining = seconds / percentDone - seconds;
			fprintf(out, " (%.2fs|%.2fs)  ", seconds, max(0.f, estRemaining));
		}
	}
	// ProgressReporter Data
	const int width, frequency, total;
	mutable int count, nPlusses;
	struct timeval start;
	string title;
	FILE *out;
};
class StatsCounter {
public:
	// StatsCounter Interface
	StatsCounter(const string &category, const string &name);
	void operator++() { ++num; }
	void operator++(int) { ++num; }
	void stat_max(StatsCounterType val) { num = max(val, num); }
	void stat_min(StatsCounterType val) { num = min(val, num); }
	operator int() { return (int)num; }
private:
	// StatsCounter Private Data
	StatsCounterType num;
};
class StatsRatio {
public:
	// StatsRatio Interface
	StatsRatio(const string &category, const string &name, bool percent = true);
	void add(int a, int b) { na += a; nb += b; }
private:
	// StatsRatio Private Data
	StatsCounterType na, nb;
};
template <class T> class MemoryArena {
public:
	// MemoryArena Interface
	MemoryArena() {
		nAvailable = 0;
	}
	~MemoryArena() { FreeAll(); }
	void FreeAll() {
		for (u_int i = 0; i < toDelete.size(); ++i)
			FreeCacheAligned(toDelete[i]);
		toDelete.erase(toDelete.begin(), toDelete.end());
		nAvailable = 0;
	}
	T *Alloc() {
		if (nAvailable == 0) {
			int nAlloc = max((unsigned int)16, 65536/sizeof(T));
			mem = (T *)AllocL2CacheAligned(nAlloc * sizeof(T));
			nAvailable = nAlloc;
			toDelete.push_back(mem);
		}
		--nAvailable;
		return mem++;
	}
	operator T *() {
		return Alloc();
	}
private:
	// MemoryArena Private Data
	T *mem;
	int nAvailable;
	vector<T *> toDelete;
};
template <class T> class ReferenceCounted {
public:
	ReferenceCounted() { nReferences = 0; }
	int nReferences;
private:
	ReferenceCounted(const ReferenceCounted &);
	ReferenceCounted &operator=(const ReferenceCounted &);
};
template <class T> class Reference {
public:
	// Reference constructors
	Reference(T *p = NULL) {
		ptr = p;
		if (ptr) ++ptr->nReferences;
	}
	Reference(const Reference<T> &r) {
		ptr = r.ptr;
		if (ptr) ++ptr->nReferences;
	}
	// Reference assignment operators
	Reference &operator=(const Reference<T> &r) {
		if (r.ptr) r.ptr->nReferences++;
		if (ptr && --ptr->nReferences == 0) delete ptr;
		ptr = r.ptr;
		return *this;
	}
	Reference &operator=(T *p) {
		if (p) p->nReferences++;
		if (ptr && --ptr->nReferences == 0) delete ptr;
		ptr = p;
		return *this;
	}
	// Reference destructor
	~Reference() {
		if (ptr && --ptr->nReferences == 0)
			delete ptr;
	}
	// Reference operators
	T *operator->() { return ptr; }
	const T *operator->() const { return ptr; }
	operator bool() const { return ptr != NULL; }
	//private:
	T *ptr;
};
struct Matrix4x4 : public ReferenceCounted<Matrix4x4> {
	// Matrix4x4 Methods
	Matrix4x4() {
		for (int i = 0; i < 4; ++i)
			for (int j = 0; j < 4; ++j)
				if (i == j) m[i][j] = 1.;
				else m[i][j] = 0.;
	}
	Matrix4x4(Float mat[4][4]) {
		memcpy(m, mat, 16*sizeof(Float));
	}
	Matrix4x4::Matrix4x4(Float t00, Float t01, Float t02, Float t03,
	                     Float t10, Float t11, Float t12, Float t13,
	                     Float t20, Float t21, Float t22, Float t23,
	                     Float t30, Float t31, Float t32, Float t33) {
		m[0][0] = t00; m[0][1] = t01; m[0][2] = t02; m[0][3] = t03;
		m[1][0] = t10; m[1][1] = t11; m[1][2] = t12; m[1][3] = t13;
		m[2][0] = t20; m[2][1] = t21; m[2][2] = t22; m[2][3] = t23;
		m[3][0] = t30; m[3][1] = t31; m[3][2] = t32; m[3][3] = t33;
	}
	Reference<Matrix4x4> Transpose() const;
	void Print(ostream &os) const {
		os << "[ ";
		for (int i = 0; i < 4; ++i) {
			os << "[ ";
			for (int j = 0; j < 4; ++j)  {
				os << m[i][j];
				if (j != 3) os << ", ";
			}
			os << " ] ";
		}
		os << " ] ";
	}
	static Reference<Matrix4x4> Mul(const Reference<Matrix4x4> &m1,
			const Reference<Matrix4x4> &m2) {
		Float r[4][4];
		for (int i = 0; i < 4; ++i)
			for (int j = 0; j < 4; ++j)
				r[i][j] = m1->m[i][0] * m2->m[0][j] +
	                              m1->m[i][1] * m2->m[1][j] +
	                              m1->m[i][2] * m2->m[2][j] +
	                              m1->m[i][3] * m2->m[3][j];
		return new Matrix4x4(r);
	}
	Reference<Matrix4x4> Inverse() const;
	Float m[4][4];
};
class StringHashTable {
public:
	// StringHashTable Methods
	u_int Hash(const string &str) const;
	void *Search(const string &key) const;
	void Add(const string &key, void *data);
private:
	// StringHashTable Private Data
	static const int NUM_BUCKETS = 1047;
	typedef vector<pair<string, void *> > ItemType;
	ItemType buckets[NUM_BUCKETS];
};
class DSO {
public:
	// DSO Methods
	DSO(const string &fname);
	~DSO();
	void *GetSymbol(const string &symname);
private:
#if defined(WIN32)
	HMODULE hinstLib;
#else
	void *hinstLib;
#endif
};
// Global Inline Functions
#ifdef NDEBUG
#define Assert(expr) ((void)0)
#else
#define Assert(expr) \
    ((expr) ? (void)0 : Severe("Assertion " #expr " failed in %s, line %d", \
                            __FILE__, __LINE__))
#endif // NDEBUG
inline Float Lerp(Float pos, Float start, Float end) {
	return (1.f - pos) * start + pos * end;
}
inline Float Clamp(Float val, Float low, Float high) {
	if (val < low) return low;
	else if (val > high) return high;
	else return val;
}

inline int Clamp(int val, int low, int high) {
	if (val < low) return low;
	else if (val > high) return high;
	else return val;
}
inline Float SmoothStep(Float min, Float max, Float value) {
	Float v = Clamp((value - min) / (max - min), 0., 1.);
	return -2.f * v * v * v + 3.f * v * v;
}
inline int Mod(int a, int b) {
    int n = int(a/b);
    a -= n*b;
    if (a < 0)
        a += b;
    return a;
}
inline Float Log2(Float x) {
	static Float invLog2 = 1.f / logf(2.f);
	return logf(x) * invLog2;
}
inline Float Radians(Float deg) { return ((Float)M_PI/180.f) * deg; }
inline Float Degrees(Float rad) { return (180.f/(Float)M_PI) * rad; }
#if (defined(__linux__) && defined(__i386__)) || defined(WIN32)
#define FAST_INT 1
#endif
#define _doublemagicroundeps	      (.5-1.4e-11)
	//almost .5f = .5f - 1e^(number of exp bit)
inline int Round2Int(double val) {
#ifdef FAST_INT
#define _doublemagic			double (6755399441055744.0)
	//2^52 * 1.5,  uses limited precisicion to floor
	val		= val + _doublemagic;
	return ((long*)&val)[0];
#else
	return int (val+_doublemagicroundeps);
#endif
}
inline int Float2Int(double val) {
#ifdef FAST_INT
	return (val<0) ?  Round2Int(val+_doublemagicroundeps) :
		   Round2Int(val-_doublemagicroundeps);
#else
	return (int)val;
#endif
}
inline int Floor2Int(double val) {
#ifdef FAST_INT
	return Round2Int(val - _doublemagicroundeps);
#else
	return (int)floorf(val);
#endif
}
inline int Ceil2Int(double val) {
#ifdef FAST_INT
	return Round2Int(val + _doublemagicroundeps);
#else
	return (int)ceilf(val);
#endif
}
inline int Log2Int(Float v) {
	return ((*(int *) &v) >> 23) - 127;
}
inline u_int RoundUpPow2(u_int v) {
	v--;
	v |= v >> 1;
	v |= v >> 2;
	v |= v >> 4;
	v |= v >> 8;
	v |= v >> 16;
	return v+1;
}
inline Float RandomFloat(Float min = 0.f, Float max = 1.f) {
	return Lerp(genrand_real1(), min, max);
}

inline unsigned long RandomInt() {
	return genrand_int32();
}
inline bool Quadratic(Float A, Float B, Float C, Float *t0, Float *t1) {
	// Find quadratic discriminant
	Float discrim = B * B - 4.f * A * C;
	if (discrim < 0.) return false;
	Float rootDiscrim = sqrtf(discrim);
	// Compute quadratic [[t]] values
	Float q;
	if (B < 0) q = -.5f * (B - rootDiscrim);
	else       q = -.5f * (B + rootDiscrim);
	*t0 = q / A;
	*t1 = C / q;
	if (*t0 > *t1) swap(*t0, *t1);
	return true;
}
inline Float ExponentialAverage(Float avg, Float val, Float alpha) {
	return (1.f - alpha) * val + alpha * avg;
}
#ifdef __GNUC__
#define RI_UNIMP() { static int first = 1 ; \
	if (first) { \
		first = 0; \
		Warning( "call to unimplemented function %s!", __PRETTY_FUNCTION__ ); } }
#else
#define RI_UNIMP() { static int first = 1 ; \
	if (first) { first = 0; Warning( "call to unimplemented function!"); } }
#endif

//PRAMOD
extern std::vector<Reference<GeometricPrimitive> > translucentBlobs;

#endif // LRT_H
