// gaussian.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "sampling.h"
#include "paramset.h"
// Gaussian Filter Declarations
class GaussianFilter : public Filter {
public:
	// GaussianFilter Interface
	GaussianFilter(Float xw, Float yw)
		: Filter(xw, yw) {
		expHalfX = expf(-halfXWidth);
		expHalfY = expf(-halfYWidth);
	}
	Float Evaluate(Float x, Float y) const;
private:
	// GaussianFilter Private Data
	Float expHalfX, expHalfY;
	// GaussianFilter Utility Functions
	static Float Gaussian(Float t, Float expHalf) {
		return max(0.f, expf(-t) - expHalf);
	}
};
// Gaussian Filter Definitions
Float GaussianFilter::Evaluate(Float x, Float y) const {
	return Gaussian(x, expHalfX) * Gaussian(y, expHalfY);
}
extern "C" Filter *CreateFilter(const ParamSet &ps, Float xw, Float yw) {
	return new GaussianFilter(xw, yw);
}
