// mitchell.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "sampling.h"
#include "paramset.h"
// Mitchell Filter Declarations
class MitchellFilter : public Filter {
public:
	MitchellFilter(Float xw, Float yw) : Filter(xw, yw) { }
	Float Evaluate(Float x, Float y) const;
	Float Mitchell1D(Float d) const;
};
// Mitchell Filter Definitions
Float MitchellFilter::Evaluate(Float x, Float y) const {
	return Mitchell1D(x * invHalfXWidth) *
		Mitchell1D(y * invHalfYWidth);
}
Float MitchellFilter::Mitchell1D(Float r) const {
	const Float B = .33333333333333333f;
	const Float C = .33333333333333333f;
	const Float ONE_SIXTH = .166666666666666666f;

	r = fabsf(r);
	if (r > 1)
		return ((-B - 6*C) * r*r*r + (6*B + 30*C) * r*r +
			(-12*B - 48*C) * r + (8*B + 24*C)) * ONE_SIXTH;
	else
		return ((12 - 9*B - 6*C) * r*r*r +
			(-18 + 12*B + 6*C) * r*r +
			(6 - 2*B)) * ONE_SIXTH;
}
extern "C" Filter *CreateFilter(const ParamSet &ps, Float xw, Float yw) {
	return new MitchellFilter(xw, yw);
}
