// sinc.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "sampling.h"
#include "paramset.h"
// Sinc Filter Declarations
class SincFilter : public Filter {
public:
	SincFilter(Float xw, Float yw) : Filter(xw, yw) { }
	Float Evaluate(Float x, Float y) const;
	Float Sinc1D(Float x) const;
};
// Sinc Filter Definitions
Float SincFilter::Evaluate(Float x, Float y) const{
	return Sinc1D(x * invHalfXWidth) *
		Sinc1D(y * invHalfYWidth);
}
Float SincFilter::Sinc1D(Float x) const {
	if (x < 1e-5) return 1;
	if (x > 1)    return 0;
	x *= M_PI;
	Float s = sinf(x) / x;
	Float blackman = .42f + .5f * cosf(x) + .08f * cosf(2*x);
	return s * blackman;
}
extern "C" Filter *CreateFilter(const ParamSet &ps, Float xw, Float yw) {
	return new SincFilter(xw, yw);
}
