#ifndef _BLOB_ELEMENT
#define _BLOB_ELEMENT

#include "lrt.h"
#include "primitives.h"

const float EPSILON=1.0e-10;

class BLOB_INTERVAL
{
 public:
  BLOB_INTERVAL()
    {
      type = 0;
      bound = 0.0f;
      ElementIndex = 0;
    }
  
  int type;
  float bound;
  int ElementIndex;
};


struct InterInfo
{
  float t0,t1;
  int index;
};

Point scalePoint(Point pt1, const Point& scale)
{
  Point pt = pt1;
  pt.x *= scale.x;
  pt.y *= scale.y;
  pt.z *= scale.z;
  return pt;
}

Point scalePointInv(Point pt1, const Point& scale)
{
  Point pt = pt1;
  pt.x /= scale.x;
  pt.y /= scale.y;
  pt.z /= scale.z;
  return pt;
}

Vector scaleVector(Vector v1, const Point& scale)
{
  Vector v = v1;
  v.x *= scale.x;
  v.y *= scale.y;
  v.z *= scale.z;
  return v;
}

Vector scaleVectorInv(Vector v1, const Point& scale)
{
  Vector v = v1;
  v.x /= scale.x;
  v.y /= scale.y;
  v.z /= scale.z;
  return v;
}

class BLOB_ELEMENT: public Primitive
{
 public:
  BBox WorldBound() const;
  bool Intersect(const Ray &r, Intersection *s) const;
  bool IntersectP(const Ray &r) const;
  
  void setBBox()
    {      
      Point len = Point(rad*scaleTrans.x,rad*scaleTrans.y,rad*scaleTrans.z);
      bbox.pMin = Point(center.x - len.x, center.y - len.y, center.z - len.z);
      bbox.pMax = Point(center.x + len.x, center.y + len.y, center.z + len.z);
    }
  
  BLOB_ELEMENT(Point c1 = Point(0,0,0),float r1 = 1.0, float coeff = 0.8)
    {
      center = c1;
      rad = r1;
      rad2 = rad*rad;
      
      c[0] = coeff / (rad2 * rad2);
      c[1] = -(2.0 * coeff) / rad2;
      c[2] = coeff;
      
      for(int i=0;i<5;i++) 
	f[i] = 0.0;

      scaleTrans = Point(1.0,1.0,1.0);
      setBBox();
    }
  
  
  BLOB_ELEMENT(Point center1,float radius, float* c1)
    {
      center = center1;
      rad = radius;
      rad2 = rad*rad;
      c[0] = c1[0];
      c[1] = c1[1];
      c[2] = c1[2];
      for(int i=0;i<5;i++) 
	f[i] = 0.0;
      scaleTrans = Point(1.0,1.0,1.0);
      setBBox();
    }
  
  Point center;
  float rad;
  float rad2;  
  float c[3];
  float f[5];

  Point scaleTrans;
  bool intersect_element(const Ray& ray, float  *tmin, float *tmax)const;
  BBox bbox;
  int index;
};

BBox BLOB_ELEMENT::WorldBound() const
{
  return bbox;
}

bool BLOB_ELEMENT::Intersect(const Ray &r, Intersection *s) const
{
  float  t1,t2;
  return intersect_element(r, &t1, &t2);
}

bool BLOB_ELEMENT::IntersectP(const Ray &r) const
{
  float  t1,t2;
  return intersect_element(r, &t1, &t2);
}

bool BLOB_ELEMENT::intersect_element(const Ray& ray, float  *tmin, float  *tmax) const
{
  float b, d, t,len;
  Vector V1;
  
  Point P = ray.O;
  Vector D = ray.D;
  
  P = scalePointInv(P,scaleTrans);
  D = scaleVectorInv(D,scaleTrans);
  
  //normalize D
  len = D.Length();
  float ilen = 1.0f/len;
  D = D*ilen;
  
  V1.x =  P.x - center.x;
  V1.y =  P.y - center.y;
  V1.z =  P.z - center.z;
  
  b = Dot(V1, D);
  t = Dot(V1, V1);
  
  d = b * b - t + rad2;
  
  if (d < EPSILON)
    {
      return false;
    }
  
  d = sqrt(d);
  
  *tmax = (- b + d)*ilen;  if (*tmax < ray.mint) { *tmax = 0.0; }
  *tmin = (- b - d)*ilen;  if (*tmin < ray.mint) { *tmin = 0.0; }
  
  if (*tmax == *tmin)
    {
      return false;
    }
  else
    {
    if (*tmax < *tmin)
      {
	d = *tmin;  *tmin = *tmax;  *tmax = d;
      }
    }
  
  return true;
}


#endif
