// area.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "light.h"
#include "primitives.h"
// AreaLight Function Definitions
AreaLight::AreaLight(bool shadows, const Transform &light2world,
		const Spectrum &power, const Reference<Shape> &s)
	: Light(shadows, light2world, power) {
	shape = s;
	area = shape->Area();
}
Spectrum AreaLight::L(const Point &Ps, const Point &Pl,
		VisibilityTester *visibility) const {
	DifferentialGeometry hit;
	Float thit;
	Vector w = Pl - Ps;
	Ray ray(Ps, w);
	if (shape->Intersect(ray, &thit, &hit)) {
		visibility->SetSegment(Ps, Pl, CastsShadows);
		return L(Pl, -w);
	}
	return 0.;
}
Spectrum AreaLight::Sample_L(const Point &P,
		const Normal &N, Float u1, Float u2,
		Vector *wo, Float *wt, bool *deltaLight,
		VisibilityTester *visibility) const {
	Normal Ns;
	Point Ps = shape->Sample(P, N, u1, u2, &Ns);
	*wo = (Ps - P).Hat();
	*wt = shape->Weight(P, N, *wo);
	*deltaLight = false;
	return L(P, Ps, visibility);
}
Spectrum AreaLight::Sample_L(const Point &P,
		Float u1, Float u2, Vector *wo, Float *wt,
		bool *deltaLight, VisibilityTester *visibility) const {
	Normal Ns;
	Point Ps = shape->Sample(P, u1, u2, &Ns);
	*wo = (Ps - P).Hat();
	*wt = shape->Weight(P, *wo);
	*deltaLight = false;
	return L(P, Ps, visibility);
}
Spectrum AreaLight::Sample_L(const Scene *scene, Float u1, Float u2,
		Float u3, Float u4, Ray *ray, Float *weight,
		bool *deltaLight) const {
	Normal Ns;
	ray->O = shape->Sample(u1, u2, &Ns);
	ray->D = UniformSampleSphere(u3, u4);
	// one sided lights? if (Dot(ray->D, Ns) < 0.) ray->D *= -1;
	*weight = shape->Weight(ray->O);
	*deltaLight = false;
	return L(ray->O, ray->D);
}
Float AreaLight::Weight(const Point &P, const Normal &N,
		const Vector &w) const {
	return shape->Weight(P, N, w);
}
Float AreaLight::Weight(const Point &P, const Vector &w) const {
	return shape->Weight(P, w);
}
Spectrum AreaLight::dE(const Point &P, const Normal &N, Vector *wo,
		VisibilityTester *visibility) const {
	Normal Ns;
	Point Ps = shape->Sample(P, N, 0.5, 0.5, &Ns);
	*wo = (Ps - P).Hat();
	return L(P, Ps, visibility) * fabsf(Dot(N, *wo)) /
		shape->Weight(P, N, *wo);
}
extern "C" Light *CreateAreaLight(bool castsShadows,
		const Transform &light2world, const ParamSet &paramSet,
		const Reference<Shape> &shape) {
	Spectrum Phi = paramSet.FindOneSpectrum("lightcolor",
		Spectrum(1.)) *	paramSet.FindOneFloat("intensity", 1.);
	Phi = paramSet.FindOneSpectrum("Phi", Phi);
	return new AreaLight(castsShadows, light2world, Phi, shape);
}
