// distantlight.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "light.h"
#include "shapes.h"
#include "scene.h"
// InfinitePointLight Classes
class InfinitePointLight : public Light {
public:
	// InfinitePointLight Methods
	InfinitePointLight(bool shadows,const Transform &light2world,
		const Spectrum &radiance, const Vector &dir);
	Spectrum dE(const Point &P, const Normal &N,
		Vector *w, VisibilityTester *) const;
	Spectrum Sample_L(const Point &P, Float u1, Float u2, Vector *wo, Float *weight,
		bool *deltaLight, VisibilityTester *visibility) const;
	Spectrum Sample_L(const Scene *scene, Float u1, Float u2,
		Float u3, Float u4, Ray *ray, Float *weight,
		bool *deltaLight) const;
	Float Weight(const Point &, const Vector &) const;
private:
	// InfinitePointLight Private Data
	Vector lightDir;
	Spectrum L;
};
// InfinitePointLight Method Definitions
InfinitePointLight::InfinitePointLight(bool shadows,
		const Transform &light2world, const Spectrum &radiance,
		const Vector &dir)
	: Light(shadows, light2world, radiance) {
	lightDir = -LightToWorld(dir).Hat();
	L = radiance;
	cout << "light direction " << lightDir << endl;
}
Spectrum InfinitePointLight::dE(const Point &P,
		const Normal &N, Vector *w, VisibilityTester *visibility) const {
	*w = lightDir;
	visibility->SetRay(P, *w, CastsShadows);
	//	if(debugflag)
	//		cout << "direction" << *w << " N=" << N << "    => " <<Dot(*w, N) << endl;
	return L * fabs(Dot(*w, N));
}
Spectrum InfinitePointLight::Sample_L(const Point &P, Float u1, Float u2,
		Vector *wo, Float *weight, bool *deltaLight,
		VisibilityTester *visibility) const {
	*wo = lightDir;
	*weight = 1;
	*deltaLight = true;
	visibility->SetRay(P, *wo, CastsShadows);
	return L;
}
Spectrum InfinitePointLight::Sample_L(const Scene *scene,
		Float u1, Float u2, Float u3, Float u4,
		Ray *ray, Float *weight, bool *deltaLight) const {
	// Choose point on disk oriented toward infinite light direction
	Point worldCenter;
	Float worldRadius;
	scene->WorldBound().BoundingSphere(&worldCenter, &worldRadius);
	Vector v1, v2;
	CoordinateSystem(lightDir, &v1, &v2);
	Float d1, d2;
	ConcentricSampleDisk(u1, u2, &d1, &d2);
	Point Pdisk = worldCenter + worldRadius * (d1 * v1 + d2 * v2);
	// Set ray origin and direction for infinite light ray
	ray->O = Pdisk + worldRadius * lightDir;
	ray->D = -lightDir;
	*deltaLight = true;
	*weight = 1.f / (M_PI * worldRadius * worldRadius);
	return L;
}
Float InfinitePointLight::Weight(const Point &, const Vector &) const {
	return 0.;
}
extern "C" Light *CreateLight(bool castsShadows,
		const Transform &light2world,
		const ParamSet &paramSet) {
	// Compute infinite light radiance
	Spectrum L = paramSet.FindOneSpectrum("lightcolor", Spectrum(1.)) *
		paramSet.FindOneFloat("intensity", 1.);
	L *= M_PI;
	L = paramSet.FindOneSpectrum("L", L);
	Point from = paramSet.FindOnePoint("from", Point(0,0,0));
	Point to = paramSet.FindOnePoint("to", Point(0,0,1));
	Vector dir = to-from;
	return new InfinitePointLight(castsShadows, light2world,
		L, dir);
}
