// goniometric.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "light.h"
#include "shapes.h"
#include "scene.h"
#include "texture.h"
// GoniometricLight Classes
class GoniometricLight : public Light {
public:
	GoniometricLight(bool shadows, const Transform &light2world,
		const Spectrum &, const string &texname);
	// GoniometricLight Member Functions
	Spectrum dE(const Point &P, const Normal &N, Vector *w, VisibilityTester *vis) const;
	~GoniometricLight() { delete mipmap; }
	Float Scale(const Vector &w) const {
		Vector wp = w;
		swap(wp.y, wp.z);
		Float theta = SphericalTheta(wp);
		Float phi = SphericalPhi(wp);
		if (mipmap)
			return mipmap->Lookup(theta / M_PI, phi / (2.f*M_PI),
				0, 0, 0, 0);
		else return 1.;
	}
	Spectrum Sample_L(const Point &P, Float u1, Float u2, Vector *wo,
		Float *weight, bool *deltaLight, VisibilityTester *visibility) const;
	Spectrum Sample_L(const Scene *scene, Float u1, Float u2,
			Float u3, Float u4, Ray *ray, Float *weight,
			bool *deltaLight) const;
	Float Weight(const Point &, const Vector &) const;
private:
	// GoniometricLight Private Data
	Point lightPos;
	Spectrum Intensity;
	MIPMap<Float> *mipmap;

};
// GoniometricLight Method Definitions
GoniometricLight::GoniometricLight(bool shadows, const Transform &light2world,
		const Spectrum &intensity, const string &texname)
	: Light(shadows, light2world, intensity) {
	lightPos = LightToWorld(Point(0,0,0));
	Intensity = intensity;
	int width, height;
	Spectrum *texels = TIFFRead(texname, &width, &height);
	if (texels) {
		Float *ftex = new Float[width*height];
		for (int i = 0; i < width*height; ++i)
			ftex[i] = texels[i].y();
		mipmap = new MIPMap<Float>(width, height, ftex);
		delete[] texels;
		delete[] ftex;
	}
	else mipmap = NULL;
}
Spectrum GoniometricLight::dE(const Point &P, const Normal &N, Vector *w,
		VisibilityTester *visibility) const {
	*w = (lightPos - P).Hat();
	visibility->SetSegment(P, lightPos, CastsShadows);
	return Intensity * Scale(-*w) * fabs(Dot(*w, N)) /
		DistanceSquared(lightPos, P);
}
Spectrum GoniometricLight::Sample_L(const Point &P, Float u1, Float u2,
		Vector *wo, Float *weight, bool *deltaLight,
		VisibilityTester *visibility) const {
	*wo = (lightPos - P).Hat();
	*weight = DistanceSquared(lightPos, P);
	*deltaLight = true;
	visibility->SetSegment(P, lightPos, CastsShadows);
	return Intensity * Scale(-*wo);
}
Spectrum GoniometricLight::Sample_L(const Scene *scene, Float u1, Float u2,
		Float u3, Float u4, Ray *ray, Float *weight,
		bool *deltaLight) const {
	ray->O = lightPos;
	ray->D = UniformSampleSphere(u1, u2);
	*weight = 1.f / (4.f * M_PI);
	*deltaLight = true;
	return Intensity * Scale(ray->D);
}
Float GoniometricLight::Weight(const Point &, const Vector &) const {
	return 0.;
}
