// bluepaint.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// BluePaint Class Declarations
class BluePaint : public Material {
public:
	BluePaint(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};
// BluePaint Method Definitions
BSDF *BluePaint::GetBSDF(const Intersection *isect) const {
	// Declare bluepaint coefficients
	static Float diffuse[3] = {  0.3094f,    0.39667f,   0.70837f  };
	static Float xy0[3] =     {  0.870567f,  0.857255f,  0.670982f };
	static Float z0[3] =      {  0.803624f,  0.774290f,  0.586674f };
	static Float e0[3] =      { 21.820103f, 18.597755f,  7.472717f };
	static Float xy1[3] =     { -0.451218f, -0.406681f, -0.477976f };
	static Float z1[3] =      {  0.023123f,  0.017625f,  0.227295f };
	static Float e1[3] =      {  2.774499f,  2.581499f,  3.677653f };
	static Float xy2[3] =     { -1.031545f, -1.029426f, -1.026588f };
	static Float z2[3] =      {  0.706734f,  0.696530f,  0.687715f };
	static Float e2[3] =      { 66.899060f, 63.767912f, 57.489181f };
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new BluePaint(displace);
}
