// brushedmetal.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// BrushedMetal Class Declarations
class BrushedMetal : public Material {
public:
	BrushedMetal(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};
// BrushedMetal Method Definitions
BSDF *BrushedMetal::GetBSDF(const Intersection *isect) const {
	// Declare brushedmetal coefficients
	static Float diffuse[3] = { 0, 0, 0 };
	static Float xy0[3] =     {  -1.11854f, -1.11845f, -1.11999f  };
	static Float z0[3] =      {   1.01272f,  1.01469f,  1.01942f  };
	static Float e0[3] =      {  15.8708f,  15.6489f,  15.4571f   };
	static Float xy1[3] =     {  -1.05334f, -1.06409f, -1.08378f  };
	static Float z1[3] =      {   0.69541f,  0.662178f, 0.626672f };
	static Float e1[3] =      { 111.267f,   88.9222f,  65.2179f   };
	static Float xy2[3] =     {  -1.01684f, -1.01635f, -1.01529f  };
	static Float z2[3] =      {   1.00132f,  1.00112f,  1.00108f  };
	static Float e2[3] =      { 180.181f,  184.152f,  195.773f    };
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(*diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new BrushedMetal(displace);
}
