// clay.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Clay Class Declarations
class Clay : public Material {
public:
	Clay(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};

// Clay Method Definitions
BSDF *Clay::GetBSDF(const Intersection *isect) const {
	// Declare clay coefficients
	static Float diffuse[3] = {   0.383626f,   0.260749f,   0.274207f };
	static Float xy0[3] =     {  -1.089701f,  -1.102701f,  -1.107603f };
	static Float z0[3] =      {  -1.354682f,  -2.714801f,  -1.569866f };
	static Float e0[3] =      {  17.968505f,  11.024489f,  21.270282f };
	static Float xy1[3] =     {  -0.733381f,  -0.793320f,  -0.848206f };
	static Float z1[3] =      {   0.676108f,   0.679314f,   0.726031f };
	static Float e1[3] =      {   8.219745f,   9.055139f,  11.261951f };
	static Float xy2[3] =     {  -1.010548f,  -1.012378f,  -1.011263f };
	static Float z2[3] =      {   0.910783f,   0.885239f,   0.892451f };
	static Float e2[3] =      { 152.912795f, 141.937171f, 201.046802f };
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new Clay(displace);
}
