// felt.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Felt Class Declarations
class Felt : public Material {
public:
	Felt(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};

// Felt Method Definitions
BSDF *Felt::GetBSDF(const Intersection *isect) const {
	// Declare felt coefficients
	static Float diffuse[3] = {  0.025865f,  0.025865f,  0.025865f};
	static Float xy0[3] =     { -0.304075f, -0.304075f, -0.304075f};
	static Float z0[3] =      { -0.065992f, -0.065992f, -0.065992f};
	static Float e0[3] =      {  3.047892f,  3.047892f,  3.047892f};
	static Float xy1[3] =     { -0.749561f, -0.749561f, -0.749561f};
	static Float z1[3] =      { -1.167929f, -1.167929f, -1.167929f};
	static Float e1[3] =      {  6.931827f,  6.931827f,  6.931827f};
	static Float xy2[3] =     {  1.004921f,  1.004921f,  1.004921f};
	static Float z2[3] =      { -0.205529f, -0.205529f, -0.205529f};
	static Float e2[3] =      { 94.117332f, 94.117332f, 94.117332f};
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new Felt(displace);
}
