// glass.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Glass Class Declarations
class Glass : public Material {
public:
	// Glass Interface
	~Glass();
	Glass(Texture<Spectrum> *r, Texture<Spectrum> *t,
			Texture<Float> *i, Texture<Float> *disp)
			: Material(disp) {
		Kr = r;
		Kt = t;
		index = i;
	}
	BSDF *GetBSDF(const Intersection *isect) const;
private:
	// Glass Private Data
	Texture<Spectrum> *Kr, *Kt;
	Texture<Float> *index;
};
// Glass Method Definitions
Glass::~Glass() {
	delete Kr;
	delete Kt;
	delete index;
}
BSDF *Glass::GetBSDF(const Intersection *isect) const {
	Spectrum R = Kr->Evaluate(isect->dgShading);
	Spectrum T = Kt->Evaluate(isect->dgShading);
	Float ior = index->Evaluate(isect->dgShading);
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	if (!R.Black())
		ret->Add(new SpecularReflection(R,
			new FresnelDielectric(1., ior)));
	if (!T.Black())
		ret->Add(new SpecularTransmission(T, 1., ior));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	SURF_TEX_F(Kr, 1.);
	SURF_TEX_F(Kt, 1.);
	SURF_TEX_F(index, 1.5f);
	SURF_TEX_S(Cs1, color);
	SURF_TEX_S(Cs2, color);
	return new Glass(new ScaleTexture<Float, Spectrum>(Kr, Cs1),
		new ScaleTexture<Float, Spectrum>(Kt, Cs2), index, displace);
}
