// marble.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Plastic Class Declarations
class Plastic : public Material {
public:
	// Plastic Interface
	~Plastic();
	Plastic(Texture<Spectrum> *kd, Texture<Spectrum> *ks,
			Texture<Float> *rough, Texture<Float> *disp)
			: Material(disp) {
		Kd = kd;
		Ks = ks;
		roughness = rough;
	}
	BSDF *GetBSDF(const Intersection *isect) const;
private:
	// Plastic Private Data
	Texture<Spectrum> *Kd, *Ks;
	Texture<Float> *roughness;
};
// Plastic Method Definitions
Plastic::~Plastic() {
	delete Kd;
	delete Ks;
	delete roughness;
}
BSDF *Plastic::GetBSDF(const Intersection *isect) const {
	Spectrum kd = Kd->Evaluate(isect->dgShading);
	BRDF *diff = new Lambertian(kd);
	Fresnel *fresnel = new FresnelDielectric(1.5f, 1.f);
	Spectrum ks = Ks->Evaluate(isect->dgShading);
	Float rough = roughness->Evaluate(isect->dgShading);
	BRDF *spec = new Microfacet(ks, fresnel, new Blinn(1.f / rough));
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(diff);
	ret->Add(spec);
	return ret;
}
// Marble Dynamic Creation Routine
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	SURF_TEX_F(Kd, 1.);
	SURF_TEX_F(Ks, 1.);
	SURF_TEX_F(roughness, 0.1f);

	float freq = geomParams.FindOneFloat("frequency",
		surfaceParams.FindOneFloat("frequency", 1.f));
	float variation = geomParams.FindOneFloat("variation",
		surfaceParams.FindOneFloat("variation", 0.5f));
	int maxOctaves = geomParams.FindOneInt("maxoctaves",
		surfaceParams.FindOneInt("maxoctaves", 8));
	Texture<Spectrum> *kd = new ScaleTexture<Float, Spectrum>(Kd,
		new MarbleTexture(maxOctaves, variation,
			new IdentityMapping3D(Scale(freq, freq, freq))));
	Texture<Spectrum> *ks = new ScaleTexture<Float, Spectrum>(Ks,
		new ConstantTexture<Spectrum>(1.));

	return new Plastic(kd, ks, roughness, displace);
}
