// matte.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Matte Class Declarations
class Matte : public Material {
public:
	// Matte Interface
	Matte(Texture<Spectrum> *kd, Texture<Float> *sig,
			Texture<Float> *disp)
		: Material(disp) {
		Kd = kd;
		sigma = sig;
		isphoton=true;
	}
	~Matte();
	BSDF *GetBSDF(const Intersection *isect) const;
private:
	// Matte Private Data
	Texture<Spectrum> *Kd;
	Texture<Float> *sigma;
};
// Matte Method Definitions
Matte::~Matte() {
	delete Kd;
	delete sigma;
}
BSDF *Matte::GetBSDF(const Intersection *isect) const {
	Spectrum r = Kd->Evaluate(isect->dgShading);
	Float sig = sigma->Evaluate(isect->dgShading);
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	if (sig == 0.)
		ret->Add(new Lambertian(r));
	else
		ret->Add(new OrenNayar(r, sig));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	SURF_TEX_S(Cs, color);
	SURF_TEX_F(Kd, 1.);
	SURF_TEX_F(sigma, 0.);
	return new Matte(new ScaleTexture<Float, Spectrum>(Kd, Cs),
		sigma, displace);
}
