// primer.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Primer Class Declarations
class Primer : public Material {
public:
	Primer(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};

// Primer Method Definitions
BSDF *Primer::GetBSDF(const Intersection *isect) const {
	// Declare primer coefficients
	static Float diffuse[3] = {  0.118230f,  0.121218f,  0.133209f};
	static Float xy0[3] =     { -0.399286f, -1.033473f, -1.058104f};
	static Float z0[3] =      {  0.167504f,  0.009545f, -0.068002f};
	static Float e0[3] =      {  2.466633f,  7.637253f,  8.117645f};
	static Float xy1[3] =     { -1.041861f, -1.100108f, -1.087779f};
	static Float z1[3] =      {  0.014375f, -0.198147f, -0.053605f};
	static Float e1[3] =      {  7.993722f, 29.446268f, 41.988990f};
	static Float xy2[3] =     { -1.098605f, -0.379883f, -0.449038f};
	static Float z2[3] =      { -0.145110f,  0.159127f,  0.173224f};
	static Float e2[3] =      { 31.899719f,  2.372852f,  2.636161f};
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new Primer(displace);
}
