// shinymetal.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// ShinyMetal Class Declarations
class ShinyMetal : public Material {
public:
	// ShinyMetal Interface
	~ShinyMetal();
	ShinyMetal(Texture<Spectrum> *ks, Texture<Float> *rough,
			Texture<Spectrum> *kr, Texture<Float> *disp)
			: Material(disp) {
		Ks = ks;
		roughness = rough;
		Kr = kr;
	}
	BSDF *GetBSDF(const Intersection *isect) const;
private:
	// ShinyMetal Private Data
	Texture<Spectrum> *Ks, *Kr;
	Texture<Float> *roughness;
};
// ShinyMetal Method Definitions
ShinyMetal::~ShinyMetal() {
	delete Ks;
	delete roughness;
	delete Kr;
}
BSDF *ShinyMetal::GetBSDF(const Intersection *isect) const {
	Spectrum spec = Ks->Evaluate(isect->dgShading);
	Float rough = roughness->Evaluate(isect->dgShading);
	Spectrum R = Kr->Evaluate(isect->dgShading);

	MicrofacetDistribution *md = new Blinn(1.f / rough);
	Spectrum k = 0.;
	Fresnel *frMf = new FresnelConductor(FresnelApproxEta(spec), k);
	Fresnel *frSr = new FresnelConductor(FresnelApproxEta(R), k);
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Microfacet(1., frMf, md));
	ret->Add(new SpecularReflection(1., frSr));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	SURF_TEX_F(Ks, 1.);
	SURF_TEX_F(Kr, 1.);
	SURF_TEX_F(roughness, 0.1f);
	SURF_TEX_S(Cs1, color);
	SURF_TEX_S(Cs2, color);
	Texture<Spectrum> *ks = new ScaleTexture<Float, Spectrum>(Ks, Cs1);
	Texture<Spectrum> *kr = new ScaleTexture<Float, Spectrum>(Kr, Cs2);
	return new ShinyMetal(ks, roughness, kr, displace);
}
