// skin.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Skin Class Declarations
class Skin : public Material {
public:
	Skin(Texture<Float> *disp) : Material(disp) { }
	BSDF *GetBSDF(const Intersection *isect) const;
};

// Skin Method Definitions
BSDF *Skin::GetBSDF(const Intersection *isect) const {
	// Declare skin coefficients
	static Float diffuse[3] = {  0.428425f,  0.301341f,  0.331054f};
	static Float xy0[3] =     { -1.131747f, -1.016939f, -0.966018f};
	static Float z0[3] =      { -1.209182f, -1.462488f, -1.222419f};
	static Float e0[3] =      {  6.421658f,  3.699932f,  3.524889f};
	static Float xy1[3] =     { -0.546570f, -0.643533f, -0.638934f};
	static Float z1[3] =      {  0.380123f,  0.410559f,  0.437367f};
	static Float e1[3] =      {  3.685044f,  4.266495f,  4.539742f};
	static Float xy2[3] =     { -0.998888f, -1.020153f, -1.027479f};
	static Float z2[3] =      {  0.857998f,  0.703913f,  0.573625f};
	static Float e2[3] =      { 64.208486f, 63.919687f, 43.809866f};
	static Spectrum xy[3] = { Spectrum(xy0), Spectrum(xy1), Spectrum(xy2) };
	static Spectrum z[3] = { Spectrum(z0), Spectrum(z1), Spectrum(z2) };
	static Spectrum e[3] = { Spectrum(e0), Spectrum(e1), Spectrum(e2) };
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new Lafortune(Spectrum(diffuse), 3, xy, xy, z, e));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new Skin(displace);
}
