// skyplastic.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"

class SkyMapping2D : public TextureMapping2D {
public:
	IdentityMapping2D(Float su = 1, Float sv = 1,
		Float du = 0, Float dv = 0);
	void Map(const DifferentialGeometry &dg, Float *s, Float *t,
		Float *dsdx, Float *dtdx,
		Float *dsdy, Float *dtdy) const;
private:
	Float su, sv, du, dv;
};

// Texture Method Definitions
SkyMapping2D::SkyMapping2D(Float _su, Float _sv,
		Float _du, Float _dv) {
	su = _su; sv = _sv;
	du = _du; dv = _dv;
}
void SkyMapping2D::Map(const DifferentialGeometry &dg,
		Float *s, Float *t, Float *dsdx, Float *dtdx,
		Float *dsdy, Float *dtdy) const {
	*s = su * dg.u + du;
	*t = sv * dg.v + dv;
	// Compute texture differentials for 2D identity mapping
	*dsdx = su * dg.dudx;
	*dtdx = sv * dg.dvdx;
	*dsdy = su * dg.dudy;
	*dtdy = sv * dg.dvdy;
}


// Plastic Class Declarations
class SkyPlastic : public Material {
public:
	// Plastic Interface
	~SkyPlastic();
	SkyPlastic(Texture<Spectrum> *kd, Texture<Float> *disp)
			: Material(disp) {
		Kd = kd;
	}
	BSDF *GetBSDF(const Intersection *isect) const;

	Texture<Spectrum> *Kd;
};
// Plastic Method Definitions
SkyPlastic::~SkyPlastic() {
	delete Kd;
}
BSDF *SkyPlastic::GetBSDF(const Intersection *isect) const {
	Spectrum kd = Kd->Evaluate(isect->dgShading);
	BRDF *diff = new Lambertian(kd);
	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(diff);
	return ret;
}
// Plastic Dynamic Creation Routine
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	return new SkyPlastic(kd, displace);
}
