// substrate.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "materials.h"
// Substrate Class Declarations
class Substrate : public Material {
public:
	// Substrate Interface
	Substrate(Texture<Spectrum> *kd, Texture<Spectrum> *ks,
			Texture<Float> *u, Texture<Float> *v, Texture<Float> *disp)
		: Material(disp) {
		Kd = kd;
		Ks = ks;
		nu = u;
		nv = v;
	}
	~Substrate();
	BSDF *GetBSDF(const Intersection *isect) const;
private:
	// Substrate Private Data
	Texture<Spectrum> *Kd, *Ks;
	Texture<Float> *nu, *nv;
};
// Substrate Method Definitions
Substrate::~Substrate() {
	delete Kd;
	delete Ks;
	delete nu;
	delete nv;
}
BSDF *Substrate::GetBSDF(const Intersection *isect) const {
	Spectrum d = Kd->Evaluate(isect->dgShading);
	Spectrum s = Ks->Evaluate(isect->dgShading);
	Float u = nu->Evaluate(isect->dgShading);
	Float v = nv->Evaluate(isect->dgShading);

	BSDF *ret = new BSDF(isect->dgShading, isect->dgGeom);
	ret->Add(new FresnelBlend(d, s, new Anisotropic(1.f/u, 1.f/v)));
	return ret;
}
extern "C" Reference<Material> CreateMaterial(const Transform &xform,
		const ParamSet &geomParams, const ParamSet &surfaceParams,
		const Spectrum &color, Texture<Float> *displace) {
	SURF_TEX_S(Cs, color);
	SURF_TEX_S(diffuse, 1.);
	SURF_TEX_S(glossy, 1.);
	SURF_TEX_F(uroughness, .1f);
	SURF_TEX_F(vroughness, .1f);
	return new Substrate(new ScaleTexture<Spectrum, Spectrum>(diffuse, Cs),
		glossy, uroughness, vroughness, displace);
}
