// geometry.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "geometry.h"
// BBox Method Definitions
BBox Union(const BBox &b, const Point &p) {
	BBox ret = b;
	ret.pMin.x = min(b.pMin.x, p.x);
	ret.pMin.y = min(b.pMin.y, p.y);
	ret.pMin.z = min(b.pMin.z, p.z);
	ret.pMax.x = max(b.pMax.x, p.x);
	ret.pMax.y = max(b.pMax.y, p.y);
	ret.pMax.z = max(b.pMax.z, p.z);
	return ret;
}
BBox Union(const BBox &b, const BBox &b2) {
	BBox ret;
	ret.pMin.x = min(b.pMin.x, b2.pMin.x);
	ret.pMin.y = min(b.pMin.y, b2.pMin.y);
	ret.pMin.z = min(b.pMin.z, b2.pMin.z);
	ret.pMax.x = max(b.pMax.x, b2.pMax.x);
	ret.pMax.y = max(b.pMax.y, b2.pMax.y);
	ret.pMax.z = max(b.pMax.z, b2.pMax.z);
	return ret;
}
BBox BIntersection(const BBox &b1, const BBox &b2) {
	BBox ret;
	ret.pMin.x = max(b1.pMin.x, b2.pMin.x);
	ret.pMin.y = max(b1.pMin.y, b2.pMin.y);
	ret.pMin.z = max(b1.pMin.z, b2.pMin.z);
	ret.pMax.x = min(b1.pMax.x, b2.pMax.x);
	ret.pMax.y = min(b1.pMax.y, b2.pMax.y);
	ret.pMax.z = min(b1.pMax.z, b2.pMax.z);
	return ret;
}
void BBox::BoundingSphere(Point *c, Float *rad) const {
	*c = .5f * pMin + .5f * pMax;
	*rad = Distance(*c, pMax);
}
bool BBox::IntersectP(const Ray &ray, Float *hitt0,
		Float *hitt1) const {
	// Initialize parametric interval
	Float t0 = ray.mint, t1 = ray.maxt;
	// Check X slab
	Float invRayDir = 1.f / ray.D.x;
	Float tNear = (pMin.x - ray.O.x) * invRayDir;
	Float tFar  = (pMax.x - ray.O.x) * invRayDir;
	// Update parametric interval
	if (tNear > tFar) swap(tNear, tFar);
	t0 = max(tNear, t0);
	t1 = min(tFar,  t1);
	if (t0 > t1) return false;
	// Check Y slab
	invRayDir = 1.f / ray.D.y;
	tNear = (pMin.y - ray.O.y) * invRayDir;
	tFar  = (pMax.y - ray.O.y) * invRayDir;
	// Update parametric interval
	if (tNear > tFar) swap(tNear, tFar);
	t0 = max(tNear, t0);
	t1 = min(tFar,  t1);
	if (t0 > t1) return false;
	// Check Z slab
	invRayDir = 1.f / ray.D.z;
	tNear = (pMin.z - ray.O.z) * invRayDir;
	tFar  = (pMax.z - ray.O.z) * invRayDir;
	// Update parametric interval
	if (tNear > tFar) swap(tNear, tFar);
	t0 = max(tNear, t0);
	t1 = min(tFar,  t1);
	if (t0 > t1) return false;
	if (hitt0) *hitt0 = t0;
	if (hitt1) *hitt1 = t1;
	return true;
}
