#ifndef HEIGHTFIELD_H
#define HEIGHTFIELD_H

#include "Array.h"
#include "Globals.h"
#include "Vectors.h"
#include "math.h"

#define MAXPERIOD 5000;

typedef VectorNd <int,3> Vector3i;


// Waves described in Vectorized Procedural Models for Natural Terrain: Waves and Islands in the Sunset, by Nelson L Max
typedef struct {
	int period;
	GLfloat dirX;
	GLfloat dirZ;
	GLfloat amplitude;
	GLfloat phaseX;
	GLfloat phaseZ;
	GLfloat *precalcheight;  // i*j*period
	GLfloat *precalcdydx;
	GLfloat *precalcdydz;
	
//	Array<GLfloat,3> precalcheight;
//	Array<GLfloat,3> precalcdydx;
//	Array<GLfloat,3> precalcdydz;
} Wave;

inline void normalize(VectorGLf &v)
{
  v /= sqrt(v.norme2());
}

class Heightfield
{
public:
	int heightw;
	int heighth;
	VectorGLf dim;
	GLfloat steph, stepw, yscale;
	GLfloat posy;		// Position of level 0 for the water
	
	std::vector<Wave> wavetab;

//	Array<GLfloat,2> heights;
//	Array<VectorGLf,2> normals;

	GLfloat *heights;
	VectorGLf *normals;
	GLfloat *colors;
	VectorGLf *refracted;
	int numwaves;

public:
	Heightfield(const Vector &size, const VectorGLf &dim, int numwaves);
	~Heightfield();
	void generateWaves(int num);
	void precalcOneWave(Wave &w);
	void calcRefracFresnel(VectorGLf playpos);
	void calcRefracInside(VectorGLf playpos);
	void display();
	void alphadisplay();
	void refracdisplay();
	void refracdisplay2();
	void update(int elapsed);
	int timeline;
	void showNormals();
};

#endif
