#include "Pool.h"
#include "World.h"
#include "Heightfield.h"
#include "Globals.h"
#include "Player.h"

Pool::Pool(int texind) {
	lookleftright=0.0f;
	lookupdown=0.0f;
	for(int i=0;i<6;i++)
		needtoredraw[i]=5;
	this->texindex=texind;
	inside=false;
}

Pool::~Pool()
{
}

void Pool::setviewer(const Player *player) { 
	lookleftright=player->lookleftright;
	lookupdown=player->lookupdown;
	playpos=player->pos-pos;
	playpos.y()+=PLAYER_RADIUS-0.5f;
	inside=player->viewinside;
}

void Pool::displayContainer(const World &world, bool minimap) const {
	glPushMatrix();
	glPushAttrib(GL_ENABLE_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT);
	glTranslatef(pos.x(),0,pos.z());
	drawHole(p, minimap);
	glPopAttrib();
	glPopMatrix();
}	

void Pool::display(const World &world, bool minimap) const {
	if(minimap) {
		glPushMatrix();
		glPushAttrib(GL_ENABLE_BIT|GL_CURRENT_BIT);
		glTranslatef(pos.x(),0,pos.z());

		glBegin(GL_QUADS);
		glNormal3f(0,1,0);
		glColor4f(.25f,.25f,.5f,.5f);
		glVertex3f(-.5f,0.0f, -.5f);
		glVertex3f(-.5f,0.0f, .5f);
		glVertex3f(.5f,0.0f, .5f);
		glVertex3f(.5f,0.0f, -.5f);
		glEnd();

		glPopAttrib();
		glPopMatrix();
		return;
	}

	glPushMatrix();
	glPushAttrib(GL_ENABLE_BIT|GL_DEPTH_BUFFER_BIT|GL_TEXTURE_BIT|GL_CURRENT_BIT);
	glTranslatef(pos.x(),0,pos.z());



	// If inside the pool !
	// Refraction only
	if(inside) {   //MBGP2
		// >>Begin of Refraction map
		glColor4f(.8f,.8f,1.0f,1.0f);

		glEnable(GL_BLEND);

		drawHole(p, minimap);	

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_CULL_FACE);
		glBegin(GL_QUADS);
		glNormal3f(0,-1,0);
		glColor4f(.25f,.25f,0.5f,1.0f);
		glVertex3f(-.5f,0.0f, -.5f);
		glVertex3f(-.5f,0.0f, .5f);
		glVertex3f(.5f,0.0f, .5f);
		glVertex3f(.5f,0.0f, -.5f);
		glEnd();

		world.heightfield->calcRefracInside(playpos);

		// >>Begin of Refraction map
		glColor4f(.8f,.8f,1,0.6f);

		glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, textures[TEXCUBEMAP]);
		glEnable(GL_TEXTURE_CUBE_MAP_EXT);

		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);

		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		
		world.heightfield->refracdisplay2();

		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glDisable(GL_TEXTURE_CUBE_MAP_EXT);
		
	// << End of refraction part
	}

	// If outside the pool !
	else {

		drawHole(p, minimap);	
 
		glEnable(GL_BLEND);
		glDisable(GL_CULL_FACE);
		glDisable(GL_TEXTURE_2D);
		glDepthMask(GL_FALSE);
				
		world.heightfield->calcRefracFresnel(playpos);
	// >>Begin of Refraction map
		glColor4f(1,1,1,1);
		
		glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, textures[TEXCUBEMAP]);
		glEnable(GL_TEXTURE_CUBE_MAP_EXT);

		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);

	//  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP);
	//  glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP);

		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
		
	//		gluSphere(quadratic,0.3,16,16);
		world.heightfield->refracdisplay();

		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glDisable(GL_TEXTURE_CUBE_MAP_EXT);
		
	// << End of refraction part


		/*
	// >> Fresnel part
		glColor4f(1,1,1,1);
		glDisable(GL_TEXTURE_2D);
		//		glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_ALPHA); MBGP3mk
		glBlendFunc(GL_ONE, GL_ONE);
 		world.heightfield->alphadisplay();
		*/
	// << End of Fresnel Part ---> included in reflection
		
		
	// >>Begin of Reflection map
	//	glBlendFunc(GL_ONE_MINUS_DST_ALPHA,GL_ONE);  MBGP3mk
		//		glColor4f(1,1,1,world.heightfield->colors[820]);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);  // Better than
																											 // before only two
																											 // renders of the
																											 // pool !!!!

	//	glBlendFunc(GL_ONE_MINUS_DST_ALPHA,GL_DST_ALPHA);
	
		glBindTexture(GL_TEXTURE_CUBE_MAP_EXT, textures[TEXCUBEMAP]);
		glEnable(GL_TEXTURE_CUBE_MAP_EXT);

		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_GEN_R);

		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);

		glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP);


		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		// Correction for the viewpoint
		glRotatef(lookleftright, 0.0f, 1.0f, 0.0f);
		glRotatef(-lookupdown, 1.0f, 0.0f, 0.0f);

		world.heightfield->display();
		
		//Restore texture matrix.
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	// << End of reflection part
	}	

	
//	glBlendFunc(GL_ONE,GL_ONE);
//	glColor4f(.25f, .25f, .5f, .5f);
//	gluSphere(quadratic,0.3,16,16);
//	world.heightfield->display();
//	world.heightfield->showNormals();


	glPopAttrib();
	glPopMatrix();
}

#define abs(x) MAX((x),-(x))
#define THR_DRAW2 10.0f
#define THR_DRAW 3.33f
#define THR_CLOSE2 3.0f
#define THR_NEAR 2.0f


// real distances
bool Pool::isinrange2(const VectorGLf &testpos) const{
	VectorGLf tester=testpos-pos;
	tester.y()=0.0f;
	return (tester.norme2()<THR_DRAW2);
}

bool Pool::isincloserange2(const VectorGLf &testpos, const VectorGLf &refpos) const{
	VectorGLf tester=testpos-refpos;
	tester.y()=0.0f;
	return (tester.norme2()<THR_CLOSE2);
}

// abs val
bool Pool::isinrange1(const Vector &c) const {
	return (abs(c.x()-pos.x())+abs(c.y()-pos.z())<=THR_DRAW);
}

void Pool::update(const VectorGLf &testpos) { 
	if(!isinrange2(testpos))
		return;
	if(isincloserange2(testpos,pos)) {
		needtoredraw[CUBE_MAP_POSITIVE_Y]=1;
		needtoredraw[CUBE_MAP_NEGATIVE_Y]=1;
	}

	if(isincloserange2(testpos,pos+VectorGLf(-1.5f, 0.0f, 0.0f)))
		needtoredraw[CUBE_MAP_POSITIVE_Z]=1;

	if(isincloserange2(testpos,pos+VectorGLf(+1.5f, 0.0f, 0.0f)))
		needtoredraw[CUBE_MAP_NEGATIVE_Z]=1;

	if(isincloserange2(testpos,pos+VectorGLf(0.0f, 0.0f, +1.5f)))
		needtoredraw[CUBE_MAP_POSITIVE_X]=1;

	if(isincloserange2(testpos,pos+VectorGLf(0.0f, 0.0f, -1.5f)))
		needtoredraw[CUBE_MAP_NEGATIVE_X]=1;
}

bool Pool::isnearpool(const VectorGLf &testpos) const{
	VectorGLf tester=testpos-pos;
	tester.y()=0.0f;
	return (tester.norme2()<THR_NEAR);
}

