// ri.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef RI_H
#define RI_H
#include "lrt.h"
// RI Function Declarations
extern const char *RiDeclare(char *name, char *type);
bool lookupType(const char *tok, int *type, int *narray, string &name);
extern void RiPixelSamples(Float x, Float y);
extern void RiFormat(int x, int y, Float aspect);
extern void RiFrameAspectRatio(Float aspect);
extern void RiScreenWindow(Float left, Float right, Float bottom,
	Float top);
extern void RiCropWindow(Float left, Float right, Float bottom, Float top);
extern void RiClipping(Float hither, Float yon);
extern void RiShutter(Float time0, Float time1);
extern void RiDepthOfField(Float fstop, Float focallen, Float focaldist);
extern void RiProjectionV(const char * name, int nArgs, const char * tokens[],
	void * params[]);
extern void RiExposure(Float gain, Float gamma);
extern void RiPixelFilter(const char *filter, Float xwidth, Float ywidth,
		int nArgs, const char * tokens[], void * params[]);
extern void RiQuantize(const char * type, int one, int minimum, int maximum,
		Float ditheramp);
extern void RiDisplayV(char *name, const char * type, const char * mode, int nArgs,
	const char * tokens[], void * parameters[]);
extern void RiSamplerV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiAcceleratorV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiSearchPath(const char *path);
extern void RiToneMap(const char *name, int n, const char *tokens[],
	void *params[]);
extern void RiSurfaceIntegratorV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiVolumeIntegratorV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiWorldBegin(), RiWorldEnd();
extern void RiTransformBegin();
extern void RiTransformEnd();
extern void RiAttributeBegin();
extern void RiAttributeEnd();
extern void RiColor(Float *Cs);
extern void RiOpacity(Float *Os);
extern void RiSurfaceV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiDisplacementV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiMotionBeginV(int N, Float times[]);
extern void RiMotionEnd();
extern void RiIdentity();
extern void RiTranslate(Float dx, Float dy, Float dz);
extern void RiTransform(Float transform[16]);
extern void RiConcatTransform(Float transform[16]);
extern void RiPerspective(Float fov);
extern void RiRotate(Float angle, Float dx, Float dy, Float dz);
extern void RiScale(Float sx, Float sy, Float sz);
extern void RiLookAt(Float ex, Float ey, Float ez, Float lx, Float ly,
	Float lz, Float ux, Float uy, Float uz);
extern void RiPolygonV(int nverts, int n, const char * tokens[],
	void * parms[]);
extern void RiGeneralPolygonV(int nloops, int nverts[], int n,
	const char * tokens[], void * parms[]);
extern void *RiLightSourceV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void *RiAreaLightSourceV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiVolumeV(const char *name, int n, const char *tokens[],
	void *params[]);
extern void RiPixelVariance(Float variation);
extern void RiTextureCoordinates(Float s1, Float t1, Float s2, Float t2,
		Float s3, Float t3, Float s4, Float t4);
extern void RiShadingRate(Float r);
extern void RiShadingInterpolation(const char * type);
extern void RiRelativeDetail(Float relativedetail);
extern void RiMatte(bool onoff);
extern void RiBound(Float bound[6]), RiDetail(Float bound[6]);
extern void RiDetailRange(Float minvis, Float lowtran,
                   Float uptran, Float maxvis);
extern void RiGeometricApproximation(const char * type, Float value);
extern void RiGeometricRepresentation(const char * type);
extern void RiSkew(Float, Float, Float, Float, Float, Float, Float);
extern void RiDeformationV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiCurvesV(const char * degree, int ncurves, int nverts[], const char * wrap,
               int n, const char * tokens[], void * params[]);
extern void RiPointsV(int npts, int n, const char * tokens[], void * params[]);
extern void RiMakeTextureV(char *pic, char *tex, const char * swrap, const char * twrap,
                    const char * filterfunc, Float swidth, Float twidth,
                    int n, const char * tokens[], void * params[]);
extern void RiMakeBumpV(char *pic, char *tex, const char * swrap, const char * twrap,
                 const char * filterfunc, Float swidth, Float twidth,
                 int n, const char * tokens[], void * params[]);
extern void RiMakeLatLongEnvironmentV(char *pic, char *tex, const char * filterfunc,
                              Float swidth, Float twidth,
                               int n, const char * tokens[], void * params[]);
extern void RiMakeCubeFaceEnvironmentV(char *px, char *nx, char *py,
                                char *ny, char *pz, char *nz,
                                char *tex, Float fov, const char * filterfunc,
                                Float swidth, Float twidth,
                                int n, const char * tokens[], void * params[]);
extern void RiMakeShadowV(char *pic, char *tex,
                   int n, const char * tokens[], void * params[]);

extern void RiColorSamples(int N, Float *nRGB, Float *RGBn);
extern void RiAtmosphereV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiInteriorV(const char * name, int n, const char * tokens[],
	void * parms[]);
extern void RiExteriorV(const char * name, int n, const char * tokens[],
       void * parms[]);

extern void RiArchiveRecord(const char * type, char *format, ...);
extern void RiProcedural(void *data, Float bound[6], void (*subdivfunc)(),
	void (*freefunc)());
extern void RiSolidBegin(const char *);
extern void RiSolidEnd();
extern void *RiObjectBegin();
extern void RiObjectEnd();
extern void RiNuPatchV(int nu, int uorder, Float *uknot,
                Float umin, Float umax, int nv, int vorder,
                Float *vknot, Float vmin, Float vmax,
                int n, const char * tokens[], void * params[]);
extern void RiTrimCurve(int nloops, int *ncurves, int *order,
                 Float *knot, Float *amin, Float *amax,
                 int *n, Float *u, Float *v, Float *w);
extern void RiBasis(Float ubasis[16], int ustep, Float vbasis[16], int vstep);
extern void RiPatchV(const char * type, int n, const char * tokens[], void * params[]);
extern void RiPatchMeshV(const char * type, int nu, const char * uwrap, int nv,
                  const char * vwrap, int n, const char * tokens[], void * params[]);
extern void RiOrientation(const char * orientation);
extern void RiReverseOrientation();
extern void RiSides(int sides);
extern void RiFrameBegin(int), RiFrameEnd();
extern void RiImagerV(const char * name, int n, const char * tokens[],
	void * params[]);
extern void RiCoordinateSystem(const char *);
extern void RiIlluminate(void *light, bool on);
extern void RiHiderV(const char * type, int n, const char * tokens[], void * parms[]);
extern void RiOptionV(const char * name, int n, const char * tokens[], void * parms[]);
extern void RiAttributeV(const char * name, int n, const char * tokens[],
	void * parms[]);
void RiSphereV(Float radius, Float zmin, Float zmax,
				 Float thetaMax, int n, const char * tokens[],
				 void * params[]);
void RiPointsPolygonsV( int npolys, int nvertices[], int vertices[],
						 int n, const char * tokens[], void * params[]);
extern void RiPointsGeneralPolygonsV(int npolys, int nloops[],
	int nverts[],	int verts[], int n, const char * tokens[],
	void * parms[]);
extern void RiDiskV(Float height, Float radius, Float tmax,
	int n, const char * tokens[], void * parms[]);
extern void RiConeV(Float height, Float radius, Float tmax,
	int n, const char * tokens[], void * parms[]);
extern void RiCylinderV(Float radius, Float zmin, Float zmax, Float tmax,
	int n, const char * tokens[], void * parms[]);
extern void RiHyperboloidV(Float point1[3], Float point2[3], Float tmax,
	int n, const char * tokens[], void * parms[]);
extern void RiParaboloidV(Float rmax, Float zmin, Float zmax, Float tmax,
	int n, const char * tokens[], void * parms[]);
extern void RiTorusV(Float majrad, Float minrad, Float phimin,
	Float phimax, Float tmax, int n, const char * tokens[],
	void * parms[]);
extern void RiGeometryV(const char * type, int n, const char * tokens[], void * params[]);
void RiSubdivisionMeshV (const char * scheme, int nfaces,
			 int nvertices[], int vertices[],
			 int ntags, const char * tags[], int nargs[],
			 int intargs[], Float floatargs[],
			 int n, const char * tokens[], void * params[]);
#endif /* RI_H */
