#include "blob.h"
#include "blob_sampler.h"
#include "paramset.h"
#include "polysolve.h"

#include "scene.h"

Spectrum Blob::getTranslucentDE(const Point& pt, const Normal& nl) const
{
	//  cout<<"Inside getTranslucentDE  "<<ObjectToWorld(elements[0]->center)<<endl;
  Point Phit = WorldToObject(pt);
  Normal Nhit = WorldToObject(nl);
  
  Spectrum L = _blobSampler->getTranslucentDE(Phit,Nhit);
  return L;
}

void Blob::createTranslucent(Scene* scene){

  for(u_int u=0; u<_blobSampler->totalSamplePoints.size(); u++){
    Spectrum E(.0);
    //create the geometry of intersection points
    
    Point Phit = ObjectToWorld(_blobSampler->totalSamplePoints[u].P);
    Normal Nhit =ObjectToWorld(_blobSampler->totalSamplePoints[u].N);
    
    /*  Vector dPdu, dPdv;
    CoordinateSystem((Vector)Nhit, &dPdu, &dPdv);
    
    Vector dNdu(0,0,0), dNdv(0,0,0);
    Float u = 0, v = 0;
    
    Intersection isect;    
    isect.dgGeom = DifferentialGeometry(ObjectToWorld(Phit), ObjectToWorld(dPdu),
					ObjectToWorld(dPdv), ObjectToWorld(dNdu),
					ObjectToWorld(dNdv), u, v, this);
    isect.dgShading = isect.dgGeom; 
    BSDF *bsdf = material->GetBSDF(&isect);
    */

    // Compute reflection by integrating over the lights
   
    Vector wi;
    for (u_int i = 0; i < scene->lights.size(); ++i) {
      VisibilityTester visibility;
      Spectrum dE = scene->lights[i]->dE(Phit, Nhit, &wi, &visibility);
      
      if (dE.Black()) continue;
      //Spectrum f = bsdf->f(-ray.D, wi);
      if (/*!f.Black() && */visibility.Unoccluded(scene))
				E += /*f*/dE * visibility.Transmittance(scene);
			//				E += /*f*/dE * visibility.Transmittance(scene)*4*M_PI;
    }
		//		if(Nhit.z>-0.579 && Nhit.z<-0.575)
		//			cout << Phit << " // " << Nhit << "      => E=" << E<<endl;
    _blobSampler->totalSamplePoints[u].dE = E; 
  }
  
  //make the tree
  _blobSampler->tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(_blobSampler->totalSamplePoints);
  cout<<"translucent map created!"<<endl;
}

bool Blob::Intersect(const Ray &r, Float *thit,
		     DifferentialGeometry *dg) const 
{
  Ray ray;
 
  WorldToObject(r, &ray);
	
  Float mult = sqrt(ray.D.x*ray.D.x + ray.D.y*ray.D.y + ray.D.z*ray.D.z);
  ray.D = Vector(ray.D.x /mult, ray.D.y /mult, ray.D.z /mult);
	
  ray.mint = ray.mint*mult;
  ray.maxt = ray.maxt*mult;

  Normal Nhit(0.0,0.0,0.0);
  Point Phit(0.0,0.0,0.0);
  
  bool hit = false;
  double blah;
  hit = All_Blob_Intersections(ray, Phit,blah);
  *thit=(float)blah;
	
  if (*thit > ray.maxt)
    return false;
	
  *thit=*thit/mult;
	

  if(hit == true)
    {
      Blob_Normal(Nhit, Phit);
    }


  /*	if(debugflag) {
	cout << "Ray org: " << ray.O << "Ray D: " << ray.D << " mult: " << mult << endl;
	cout << "HIT ? " << hit << ",   Phit: " << Phit << ",
	Nhit: " << Nhit << ",  thit:" << *thit << " transformed
	"<<ObjectToWorld(Phit) << endl <<endl;
	}
  */
//hit = test(ray,Phit,Nhit,thit);

// Initialize Differential geometry if there was an intersection
  if (hit){
    Vector dPdu, dPdv;
    CoordinateSystem((Vector)Nhit, &dPdu, &dPdv);
    
    Vector dNdu(0,0,0), dNdv(0,0,0);
    Float u = 0, v = 0;
  
    *dg = DifferentialGeometry(ObjectToWorld(Phit), ObjectToWorld(dPdu),
			     ObjectToWorld(dPdv), ObjectToWorld(dNdu),
			       ObjectToWorld(dNdv), u, v, this);
  }
  
  return hit;
}

bool Blob::IntersectP(const Ray &r) const {
  /*  Ray ray;
      WorldToObject(r, &ray);
      ray.D = HAT(ray.D);
      return kdtree->IntersectP(ray);
  */
  
  DifferentialGeometry dg;
  Float thit;
  return Intersect(r, &thit, &dg);  
}

Float Blob::Area() const 
{
  // It's not necessary to implement this method for this assignment.
  return 0.;
}

extern "C" Reference<Shape>CreateShape(const Transform &obj2world, 
				       const ParamSet &params)
{
  BBox bbox(params.FindOnePoint("p0", Point(0,0,0)),
	    params.FindOnePoint("p1", Point(1,1,1)));
  string s;
  s=params.FindOneString("file", string("MakeSnow/data.blob"));
  return new Blob(obj2world, bbox, s);
}

void Blob::element_normal(Normal& Result, Point P, int ElementIndex) const
{
  blobFloat val, dist;
  Vector V1;
  Point PP;

  PP = scalePointInv(P,elements[ElementIndex]->scaleTrans);
  
  V1 = PP - elements[ElementIndex]->center;
  
  dist = Dot(V1, V1);
  
  if(dist <= elements[ElementIndex]->rad2)
    {
      val = -2.0 * elements[ElementIndex]->c[0] * dist - elements[ElementIndex]->c[1];
      V1 = scaleVectorInv(V1,elements[ElementIndex]->scaleTrans);
      Result += val*Normal(V1);
    }
}

void Blob::Blob_Normal(Normal& Result, Point IPoint) const
{
  int i;
  //  int size;
  blobFloat /*dist,*/ val;
 
  
  /* For each component that contributes to this point, add its bit to the normal */
  
  for (i = 0; i < _noc; i++)
    {
      element_normal(Result, IPoint, i);
    }
   
  val = Dot(Result, Result);

  if (val == 0.0)
    {
      Result = Normal(1.0, 0.0, 0.0);
    }
  /*
    else
    {
    // Normalize normal vector. 
    val = 1.0 / sqrt(val);
    Result *= val;
    }
  */
}

#include "polynomial.h"
#include <vector>
#include <algorithm>
#include "polysolve.h"

bool Blob::All_Blob_Intersections(Ray& ray, Point& Ipt, blobFloat &dist) const
{
  int i, j, cnt;
  int root_count, in_flag;
 
  blobFloat t0, t1, t2, c0, c1, c2, len, start_dist;
  blobFloat *fcoeffs;
  blobFloat coeffs[5];

  blobFloat roots[4];
  Point P,PP;
  Vector D, V1, DD;
  //  Point IPoint;
  
  blobFloat newcoeffs[5], dk[5];
  blobFloat l, w;

  P = ray.O;
  D = ray.D;
  len = 1.0;
  
  if ((cnt = findIntervals(ray)) == 0)
    {
      return false;
    }
  
  if ((start_dist = intervals[0].bound) < Small_Tolerance)
    {  
      start_dist = 0.0;
    }
  
  for (i = 0; i < cnt; i++)
    {
      intervals[i].bound -= start_dist;
    }

  
  /* Get the new starting point. */
  
  P = ray(start_dist);//VAddScaledEq(P, start_dist, D);
  
  /* Clear out the coefficients. */
  
  coeffs[0] =
    coeffs[1] =
    coeffs[2] =
    coeffs[3] = 0.0;
  coeffs[4] = - _threshold;
  
  /*
   * Step through the list of intersection points, adding the
   * influence of each component as it appears. 
   */
  
  fcoeffs = NULL;
  for (i = in_flag = 0; i < cnt; i++)
    {
      if(intervals[i].type == 0)
	{	  
	  /*
	   * Something is just starting to influence the ray, so calculate
	   * its coefficients and add them into the pot. 
	   */
	  
	  in_flag++;
	  
	  int index = intervals[i].ElementIndex; 
	
	  PP = scalePointInv(P,elements[index]->scaleTrans);
	  DD = scaleVectorInv(D,elements[index]->scaleTrans);
	  
	  V1 = PP - elements[index]->center;
	  
	  t0 = Dot(V1, V1);
	  t1 = Dot(V1, DD);
	  t2 = Dot(DD, DD);

	  c0 = elements[index]->c[0];
	  c1 = elements[index]->c[1];
	  c2 = elements[index]->c[2];
	  
	  fcoeffs = &(elements[index]->f[0]);
        
	  fcoeffs[0] = c0 * t2 * t2;
          fcoeffs[1] = 4.0 * c0 * t1 * t2;
          fcoeffs[2] = 2.0 * c0 * (2.0 * t1 * t1 + t0 * t2) + c1 * t2;
          fcoeffs[3] = 2.0 * t1 * (2.0 * c0 * t0 + c1);
          fcoeffs[4] = t0 * (c0 * t0 + c1) + c2;

	  /*
	    fcoeffs[0] = c0;
	    fcoeffs[1] = 4.0 * c0 * t1;
	    fcoeffs[2] = 2.0 * c0 * (2.0 * t1 * t1 + t0) + c1;
	    fcoeffs[3] = 2.0 * t1 * (2.0 * c0 * t0 + c1);
	    fcoeffs[4] = t0 * (c0 * t0 + c1) + c2;	    
	  */

	  for (j = 0; j < 5; j++)
	    {
	      coeffs[j] += fcoeffs[j];
	    }
	}
      else
	{
	  /* 
	   * We are losing the influence of a component -->
	   * subtract off its coefficients. 
	   */

	  fcoeffs = elements[intervals[i].ElementIndex]->f;

	  for (j = 0; j < 5; j++)
	    {
	      coeffs[j] -= fcoeffs[j];
	    }
	  
	  /* If no components are currently affecting the ray ---> skip ahead. */
	
	  if (--in_flag == 0)
	    {
	      continue;
	    }
	}
      
      /*
       * If the following intersection lies close to the current intersection
       * then first add/subtract next region before testing. [DB 7/94] 
       */
      
      if ((i + 1 < cnt) && (fabs(intervals[i].bound - intervals[i + 1].bound) < EPSILON))
	{
	  continue;
	}
      
      /*
       * Transform polynomial in a way that the interval boundaries are moved
       * to 0 and 1, i. e. the roots of interest are between 0 and 1. [DB 10/94]
       */
    
      l = intervals[i].bound;
      w = intervals[i+1].bound - l;
      
      newcoeffs[0] = coeffs[0] * w * w * w * w;
      newcoeffs[1] = (coeffs[1] + 4.0 * coeffs[0] * l) * w * w * w;
      newcoeffs[2] = (3.0 * l * (2.0 * coeffs[0] * l + coeffs[1]) + coeffs[2]) * w * w;
      newcoeffs[3] = (2.0 * l * (2.0 * l * (coeffs[0] * l + 0.75 * coeffs[1]) + coeffs[2]) + coeffs[3]) * w;
      newcoeffs[4] = l * (l * (l * (coeffs[0] * l + coeffs[1]) + coeffs[2]) + coeffs[3]) + coeffs[4];
    
      /* Calculate coefficients of corresponding bezier curve. [DB 10/94] */

      dk[0] = newcoeffs[4];
      dk[1] = newcoeffs[4] + 0.25 * newcoeffs[3];
      dk[2] = newcoeffs[4] + 0.50 * (newcoeffs[3] + newcoeffs[2] / 12.0);
      dk[3] = newcoeffs[4] + 0.50 * (0.375 * newcoeffs[3] + newcoeffs[2] + 0.125 * newcoeffs[1]);
      dk[4] = newcoeffs[4] + newcoeffs[3] + newcoeffs[2] + newcoeffs[1] + newcoeffs[0];
    
      /*
       * Skip this interval if the ray doesn't intersect the convex hull of the
       * bezier curve, because no valid intersection will be found. [DB 10/94]
       */
      
      if (((dk[0] >= 0.0) && (dk[1] >= 0.0) && (dk[2] >= 0.0) && (dk[3] >= 0.0) && (dk[4] >= 0.0)) ||
	  ((dk[0] <= 0.0) && (dk[1] <= 0.0) && (dk[2] <= 0.0) && (dk[3] <= 0.0) && (dk[4] <= 0.0)))
	{
	  continue;
	}


      
      /*
       * Now we could do bezier clipping to find the roots
       * but I have no idea how this works. [DB 2/95]
       */
    
      
      /* Solve polynomial. */
      
      //      blobFloat temp[5];
      //for(int u=0; u<5; u++)
      //temp[u] = coeffs[u];
      
      //coeffs.RootsDegree4(root_count, roots);
      
      //sort the roots
      //std::vector<blobFloat> sorted_roots;
      //for(int u=0; u<root_count; u++)
      //	sorted_roots.push_back(roots[u]);
      //std::sort(sorted_roots.begin(),sorted_roots.end());
      
      //      cout<<root_count<<endl;
      root_count = Solve_Polynomial(4, coeffs, roots, 1, 1.0e-11);
      
      //cout<<"h : "<<root_count<<endl;
      /* See if any of the roots are valid. */
      
      /*if(debugflag) {
	cout << "Component number: " << i << ",  roots: " << root_count << endl;
	cout << "intervals[i] " << intervals[i].bound << "//  intervals[i+1] " <<intervals[i+1].bound << endl;
	}*/
      for (j = 0; j < root_count; j++)
	{
	  dist = roots[j];
					
	  /*
	   * First see if the root is in the interval of influence of
	   * the currently active components.
					 
	  */
	  //	  if(debugflag)
	  // cout << "start_dist: " << start_dist << "  dist: " <<dist <<endl; 
					
	  if ((dist >= intervals[i].bound) &&
	      (dist <= intervals[i+1].bound))
	    {
	      /* Correct distance. */
							
	      dist = (dist + start_dist) / len;
							
	      //if ((dist > DEPTH_TOLERANCE) && (dist < Max_Distance))
	      //							if ((dist > ray.mint) && (dist < ray.maxt))
	      if ((dist > /*1*/ray.mint) && (dist < ray.maxt))
		{
		  Ipt = ray(dist);
		  return true;
		  
		  /*
		    if(Point_In_Clip(IPoint, Object->Clip))
		    {
		    push_entry(dist, IPoint, Object, Depth_Stack);
		  
		    Intersection_Found = true;
		    }
		  */
		}
	    }
	}
      
      /*
       * If the blob isn't used inside a CSG and we have found at least
       * one intersection then we are ready, because all possible intersections
       * will be further away (we have a sorted list!). [DB 7/94]
       */
      
      /*
	if (!(Blob->Type & IS_CHILD_OBJECT) && (Intersection_Found))
	{
	break;
	}
      */
    }
  
  /*  
      if (Intersection_Found)
      {
      Increase_Counter(stats[Ray_Blob_Tests_Succeeded]);
      }
      return (Intersection_Found);
  */
  return false;
}

Blob::Blob(const Transform &o2w, const BBox &ob, const string &s)
  : Shape(o2w),_noc(1) 
{
  filen=s;
  
  //  if(readBlob(s.c_str()) == false) 
  // exit(1);
  
  intervals = new BLOB_INTERVAL[2*_noc]; 
  _threshold = 0.6;
  
  MakeBlob();   // TO BE REMOVED
  kdtree = new MyKdTreeAccelerator(elements);
  objBound=kdtree->bounds;
  
  //sampler
  _blobSampler = new BlobSampler(this);
}

Blob::~Blob()
{
  //  writeBlob("MakeSnow/data.blob");
  // delete[] 
  delete[] intervals;
  delete kdtree;
  delete _blobSampler;
}

void Blob::MakeBlob()
{
  srand(0);

  BLOB_ELEMENT* element = new BLOB_ELEMENT(Point(0,0,0),1,1);
  elements.push_back(element);
	elements[0]->scaleTrans = Point(1,0.5,1);
  elements[0]->setBBox();
  elements[0]->index = 0;

  /*  cout<<elements[0]->c[0]<<endl;

  BLOB_ELEMENT* element1 = new BLOB_ELEMENT(Point(0.5,0.5,0),30,1);
  elements.push_back(element1);
  elements[1]->scaleTrans = Point(1,1,1);
  elements[1]->setBBox();
  elements[1]->index = 1;
  
  cout<<elements[1]->c[0]<<endl;
  */

    /*
  for(int i=0; i<_noc; i++)
    {
      blobFloat x = 0.01*(rand()%200 - 100);
      blobFloat y = 0.01*(rand()%200 - 100);
      
      BLOB_ELEMENT* element = new BLOB_ELEMENT(Point(x,y,0),0.1,1);      
      element->scaleTrans = Point(1,1,1.3);
      element->setBBox();
      element->index = i;
      
      elements.push_back(element);
    }  
    */
}

BBox Blob::ObjectBound() const 
{
  return objBound;
}

void memmove(void *dest, void  *src, size_t length)
{
  char *csrc =(char *)src;
  char *cdest=(char *)dest;
  
  if (csrc < cdest && csrc + length >= cdest)
    {
      size_t size = cdest - csrc;
      while (length > 0)
	{
	  memcpy(cdest + length - size, csrc + length - size, size);
	  
	  length -= size;
	  
	  if (length < size)
	    size = length;
	}
    }
  else if (cdest < csrc && cdest + length >= csrc)
    {
      char *new_dest = cdest;
      size_t size = csrc - cdest;

      while (length > 0)
	{
	  memcpy(new_dest, csrc, length);

	  new_dest += size;
	  csrc += size;
	  length -= size;

	  if (length < size)
	    size = length;
	}
    }
  else
    {
      memcpy(cdest, csrc, length);
    }
  //return cdest;
}

void Blob::insert_hit(int ElementIndex/*BLOB_ELEMENT *Element*/, blobFloat t0,blobFloat  t1, int* cnt)const
{
  int k;
  
  intervals[*cnt].type    = 0;//ENTERING;
  intervals[*cnt].bound   = t0;
  intervals[*cnt].ElementIndex = ElementIndex;
  
  for (k = 0; t0 > intervals[k].bound && k < *cnt; k++);
  
  if (k < *cnt)
    {
      //cout<<"we go here"<<endl;
      /*
       * This hit point is smaller than one that already exists -->
       * bump the rest and insert it here.
       */
      
      memmove(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(BLOB_INTERVAL));
    
      /* We are entering the component. */
      
      intervals[k].type    = 0;//ENTERING;
      intervals[k].bound   = t0;
      intervals[k].ElementIndex = ElementIndex;
      
      (*cnt)++;
      
      /* We are exiting the component. */
      
      intervals[*cnt].type    = 1;//EXITING;
      intervals[*cnt].bound   = t1;
      intervals[*cnt].ElementIndex = ElementIndex;
      
      for (k = k + 1; t1 > intervals[k].bound && k < *cnt; k++);
      
      if (k < *cnt)
	{
	  memmove(&intervals[k+1], &intervals[k], (*cnt-k)*sizeof(BLOB_INTERVAL));
	  
	  intervals[k].type    = 1;//EXITING;
	  intervals[k].bound   = t1;
	  intervals[k].ElementIndex = ElementIndex;
	}

      (*cnt)++;
    }
  else
    {
      /* Just plop the start and end points at the end of the list */
      
      (*cnt)++;
      
      /* We are exiting the component. */
      
      intervals[*cnt].type    =  1;//EXITING;
      intervals[*cnt].bound   = t1;
      intervals[*cnt].ElementIndex = ElementIndex;
      
      (*cnt)++;
    }
}

int Blob::findIntervals(Ray& ray) const
{
  //int i;
  int cnt;
  //  blobFloat t0, t1;
  
  cnt = 0;
  
  // MBGP !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!111
  
  std::vector<InterInfo> intersections;
  if(kdtree->IntersectNew(ray,intersections) == true)
    for (unsigned int j = 0; j < intersections.size(); j++)
      {
	insert_hit(intersections[j].index, intersections[j].t0,
		   intersections[j].t1,&cnt);
      }
  /*
  
  for (i = 0; i < _noc; i++)
  {
  if(elements[i]->intersect_element(ray, &t0, &t1))
  {
  insert_hit(i, t0, t1,&cnt);
  }
  }
  */
  //  if(cnt != cnt1)
  //cout<<cnt<<"....."<<endl;
  

  return (cnt);
}

bool Blob::test(Ray& ray, Point& Phit, Normal& Nhit, blobFloat* thit) const
{
  blobFloat tmin,tmax;

  bool hit = false;
  bool hit1 = elements[0]->intersect_element(ray, &tmin, &tmax);
  
  //if(hit1 == false)
  //hit = intersect_element(1, ray, ray.mint, &tmin, &tmax);
  
  hit = hit || hit1;

  //  cout<<hit;

  if(hit == false) 
    return false;
  
  blobFloat t = 1000.0;
  if(tmin > EPSILON && tmin < t)
    t = tmin;
  
  if(tmax > EPSILON && tmax < t)
    t = tmax;
  
  if(t < 1000.0)
    {
      if(t> ray.mint && t<ray.maxt)
	{
	  Phit = ray(t);
	  
	  Point PP = scalePointInv(Phit,elements[0]->scaleTrans);
	  Vector V1 = PP - elements[0]->center;
	  
	  Vector tvec(V1.x, V1.y, V1.z);
	  Nhit = Normal(HAT(tvec));

	  if(objBound.Inside(Phit) == true)
	    {
	      hit = true;
	      *thit = t;
	    }
	}
    }
  return true;
}

bool Blob::readBlob(const char* filename)
{
  cout<<filename<<endl;
  FILE* file;
  if((file = fopen(filename,"r")) == NULL)
    {
      cout<<"Blob File Doesn't exist!"<<endl;
      return false;
    }
  
  fscanf(file,"%d\n",&_noc);
  
  //elements = new BLOB_ELEMENT[_noc];
  
  for(int i=0; i<_noc; i++)
    {
      float blah[10];

      fscanf(file,"%f %f %f %f %f %f %f %f %f %f\n",&blah[0],&blah[1],&blah[2],&blah[3],&blah[4],&blah[5],&blah[6],&blah[7],&blah[8],&blah[9]);
			
      BLOB_ELEMENT* element = new BLOB_ELEMENT(Point(blah[0],blah[1],blah[2]),blah[3],1);
      elements.push_back(element);
      
      elements[i]->scaleTrans.x=blah[7];
      elements[i]->scaleTrans.y=blah[8];
      elements[i]->scaleTrans.z=blah[9];
      
      elements[i]->setBBox();
      elements[i]->index = i;
      
    }
  
  cout<<"noc   "<<_noc<<endl;
  fclose(file);
  return true;
}

bool MyKdTreeAccelerator::IntersectNew(const Ray &ray,std::vector<InterInfo>& intersections)  const
{
  intersections.clear();

  // Compute initial parametric range of ray inside kd tree extent
  Float kd_tmin, kd_tmax;
	
  /*  float mint = ray.mint;
      float maxt = ray.maxt;
  */
	
  if (!bounds.IntersectP(ray, &kd_tmin, &kd_tmax))
    {
      //      ray.mint = mint;
      //ray.maxt = maxt;
      
      return false;
    }

  //  if(maxt != ray.maxt)
  // cout<<maxt<<"  "<<ray.maxt<<endl;
	
  //ray.mint = mint;
  //ray.maxt = maxt;
  
  kd_tmax *= 1.001f;
  
  // Prepare to traverse kd-tree for ray
  int rayId = curMailboxId++;
  const MyKdAccelNode *node = root;
  
  Vector invDir(1.f/ray.D.x, 1.f/ray.D.y, 1.f/ray.D.z);
  
#define MAX_TODO 64
  MyKdToDo todo[MAX_TODO];
  int todoPos = 0;
  bool hit = false;

  while (node != NULL)
    {
      // Process kd tree node [[node]] for ray traversal
      // Update kd-tree traversal statistics
      static StatsCounter nodesTraversed("Acceleration",
					 "Number of kd-tree nodes traversed by normal rays");
      ++nodesTraversed;
      // Bail out if we found a hit closer than the current node
      
      //if (ray.maxt < tmin) break;
      
      if (node->isLeaf) 
	{
	  // Check for intersections inside leaf node
	  u_int nPrimitives = node->nPrimitives;
	  MailboxPrim **prims = node->u.primitives;
	  
	  for (u_int i = 0; i < nPrimitives; ++i)
	    {
	      MailboxPrim *mp = prims[i];
	      if (mp->lastMailboxId != rayId) 
		{
		  mp->lastMailboxId = rayId;
		  
		  blobFloat t0,t1;
		  if (mp->primitive->intersect_element(ray,&t0,&t1))
		    {
		      hit = true;
		      InterInfo temp;
		      temp.t0 = t0;
		      temp.t1 = t1;
		      temp.index = mp->primitive->index;
		      intersections.push_back(temp);
		    }
		}
	    }
	  
	  // Grab next node to process from todo list
	  if (todoPos > 0) 
	    {
	      --todoPos;
	      node = todo[todoPos].node;
	      kd_tmin = todo[todoPos].tmin;
	      kd_tmax = todo[todoPos].tmax;
	    }
	  else
	    break;
	}
      else 
	{
	  // Process kd tree interior node
	  // Compute distance along ray to split plane
	  Float tplane;
	  if (node->axis == MYSPLIT_X)
	    tplane = (node->split - ray.O.x) * invDir.x;
	  else if (node->axis == MYSPLIT_Y)
	    tplane = (node->split - ray.O.y) * invDir.y;
	  else
	    tplane = (node->split - ray.O.z) * invDir.z;
	  // Get near and far child pointers for ray
	  MyKdAccelNode *nearChild, *farChild;
	  bool zeroIsNear;
	  if (node->axis == MYSPLIT_X)
	    zeroIsNear = (ray.O.x <= node->split);
	  else if (node->axis == MYSPLIT_Y)
	    zeroIsNear = (ray.O.y <= node->split);
	  else
	    zeroIsNear = (ray.O.z <= node->split);
	  if (zeroIsNear)
	    {
	      nearChild = node->u.children[0];
	      farChild  = node->u.children[1];
	    }
	  else 
	    {
	      nearChild = node->u.children[1];
	      farChild  = node->u.children[0];
	    }

	  // Advance to next child node, possibly enqueue far child
	  if (tplane > kd_tmax || tplane < 0)
	    node = nearChild;
	  else if (tplane < kd_tmin)
	    node = farChild;
	  else
	    {
	      if(!nearChild) 
		{
		  node = farChild;
		  kd_tmin = tplane;
		}
	      else 
		{
		  if (farChild)
		    {
		      // Enqueue [[farChild]] in todo list
		      todo[todoPos].node = farChild;
		      todo[todoPos].tmin = tplane;
		      todo[todoPos].tmax = kd_tmax;
		      ++todoPos;
		      Assert(todoPos < MAX_TODO);
		    }
		  node = nearChild;
		  kd_tmax = tplane;
		}
	    }
	  if (!node)
	    {
	      // Grab next node to process from todo list
	      if (todoPos > 0)
		{
		  --todoPos;
		  node = todo[todoPos].node;
		  kd_tmin = todo[todoPos].tmin;
		  kd_tmax = todo[todoPos].tmax;
		}
	      else
		break;
	    }
	}
    }
  return hit;
}

// MyMyKdTreeAccelerator Method Definitions
MyKdTreeAccelerator::MyKdTreeAccelerator(const vector<Reference<BLOB_ELEMENT> >&prims)
{
  // Refine all [[prims]] until they are intersectable
  vector<Reference<BLOB_ELEMENT> > prefined, todo = prims;

  while (todo.size()) 
    {
      Reference<BLOB_ELEMENT> prim = todo.back();
      todo.pop_back();
      if (prim->CanIntersect())
	prefined.push_back(prim);
      else
	cout<<"Refine not defined!! : mykdtree.cc"<<endl;
      //	prim->Refine(todo);
    }
  
  // Initialize mailboxes for [[MyKdTreeAccelerator]]
  curMailboxId = 0;
  mailboxPrims = new MailboxPrim[prefined.size()];
  for (u_int i = 0; i < prefined.size(); ++i)
    mailboxPrims[i].primitive = prefined[i];
  // Set up memory pools for kd tree nodes
  int numTop = (1 << MAX_TOP_DEPTH) - 1;
  topNodes = (MyKdAccelNode *)AllocL2CacheAligned(numTop * sizeof(MyKdAccelNode));
  allocedNodeBlocks.push_back(topNodes);
  for (int i = 0; i < MAX_TOP_DEPTH; ++i)
    topNodeOffset[i] = 0;
  lowerNodes = NULL;
  lowerNodePos = LOWER_NODE_ALLOC_SIZE;
  // Build kd tree for accelerator
  vector<BBox> primBounds;
  vector<int> primNums;
  primBounds.reserve(prefined.size());
  primNums.reserve(prefined.size());
  for (u_int i = 0; i < prefined.size(); ++i) 
    {
      BBox b = prefined[i]->WorldBound();
      bounds = Union(bounds, b);
      primBounds.push_back(b);
      primNums.push_back(i);
    }
 
  
  buildTree(&root, bounds, mailboxPrims, primBounds, primNums, 0);
}

MyKdTreeAccelerator::~MyKdTreeAccelerator() 
{
  delete[] mailboxPrims;
  // Free memory pools for [[KdTreeNodes]]
  for (u_int i = 0; i < allocedNodeBlocks.size(); ++i)
    FreeCacheAligned(allocedNodeBlocks[i]);
}

void MyKdTreeAccelerator::buildTree(MyKdAccelNode **node, const BBox &nodeBounds,
				    MailboxPrim *mailboxPrims, const vector<BBox> &allPrimBounds,
				    const vector<int> &primNums, int depth)
{
  if (!primNums.size()) 
    {
      *node = NULL;
      return;
    }

  // Initialize leaf node if termination criteria met
  if (primNums.size() == 1 ||
      (primNums.size() < 5 && depth > 10) ||
      depth > 10)
    {
      //      cout<<"finished----"<<endl;
      *node = allocNode(depth, primNums, nodeBounds);
      return;
    }
  // Initialize interior node and continue recursion
  // Choose split axis for interior node
  int nextAxis;
  Vector diag = nodeBounds.pMax - nodeBounds.pMin;
  if (diag.x > diag.y && diag.x > diag.z) nextAxis = MYSPLIT_X;
  else if (diag.y > diag.z) nextAxis = MYSPLIT_Y;
  else nextAxis = MYSPLIT_Z;
  // Compute node split position and allocate interior [[MyKdAccelNode]]
  BBox bounds0 = nodeBounds, bounds1 = nodeBounds;
  Float tsplit;
  if (nextAxis == MYSPLIT_X)
    {
      tsplit = Lerp(.5f, nodeBounds.pMin.x, nodeBounds.pMax.x);
      bounds0.pMax.x = bounds1.pMin.x = tsplit;
    }
  else if (nextAxis == MYSPLIT_Y) 
    {
      tsplit = Lerp(.5f, nodeBounds.pMin.y, nodeBounds.pMax.y);
      bounds0.pMax.y = bounds1.pMin.y = tsplit;
    }
  else
    {
      tsplit = Lerp(.5f, nodeBounds.pMin.z, nodeBounds.pMax.z);
      bounds0.pMax.z = bounds1.pMin.z = tsplit;
    }
  *node = allocNode(depth, nextAxis, tsplit);
  // Classify primitives with respect to split
  vector<int> prims0, prims1;
  for (u_int i = 0; i < primNums.size(); ++i) 
    {
      int primNum = primNums[i];
      if (bounds0.Overlaps(allPrimBounds[primNum]))
	prims0.push_back(primNum);
      if (bounds1.Overlaps(allPrimBounds[primNum]))
	prims1.push_back(primNum);
    }
  // Recursively initialize children nodes
  buildTree(&((*node)->u.children[0]), bounds0, mailboxPrims, allPrimBounds,
	    prims0, depth+1);
  buildTree(&((*node)->u.children[1]), bounds1, mailboxPrims, allPrimBounds,
	    prims1, depth+1);
}

bool Blob::IntersectBlob(Ray& ray, Point& Ipt, Normal& Nhit) const
{
  //  cout<<"Origin: "<<ray.O <<"  Dir: "<<ray.D<<endl;
  
  double t; 
  bool hit = All_Blob_Intersections(ray, Ipt, t);
  
  if (t > ray.maxt)
    return false;
	
  if(hit == true)
    {
      Ipt = ray(t);
      Blob_Normal(Nhit, Ipt);
    }

  return hit;
}


/*void Blob::writeBlob(char* filename)
{
//  FILE* file = fopen(filename,"w");
  fprintf(file,"%d\n",_noc);
  
  for(int i=0; i<_noc; i++)
    {
      fprintf(file,"%f %f %f %f %f %f %f %f %f %f\n",elements[i]->center.x,elements[i]->center.y,elements[i]->center.z,elements[i]->rad,elements[i]->c[0],elements[i]->c[1],elements[i]->c[2],elements[i]->scaleTrans.x,elements[i]->scaleTrans.y,elements[i]->scaleTrans.z);
    }
  
  fclose(file);
}
*/
