#ifndef _BLOB_SAMPLER_H
#define _BLOB_SAMPLER_H

#include "blob.h"
#include "mc.h"
#include <vector>

struct SubsurfaceData 
{
  SubsurfaceData() { }
  SubsurfaceData(const DifferentialGeometry &dg, const Spectrum &L,
		 const Vector &w)
    : P(dg.P), N(dg.Nn), dE(L), wi(w) { }
  Point P;
  Normal N;
  Spectrum dE;
  Vector wi;
};

struct CloseSubsurface {
  CloseSubsurface(const SubsurfaceData *d = NULL, Float md2 = HUGE_VAL) {
    data = d;
    maxDist2 = md2;
  }
  bool operator<(const CloseSubsurface &p2) const {
    return maxDist2 < p2.maxDist2;
  }
  const SubsurfaceData *data;
  Float maxDist2;
};

struct SubsurfaceProcess {
  SubsurfaceProcess(u_int mp, const Point &p, const Normal &n);
  void operator()(const SubsurfaceData &photon, Float &maxDist) const;

  const Point &P;
  const Normal &N;


  mutable vector<CloseSubsurface> subsurfacePts;
  u_int maxSubsurfaces;
};

Spectrum BlobSampler::getTranslucentDE(const Point& Phit, const Normal& Nhit) const
{
  //  cout<<Phit<<"  "<<Nhit<<endl;
  float oneby4pi = 1.0/(4*M_PI);
  Spectrum L(0.0);

  float md2 =  100*lu;
	//float md =  10*lu;

  int Nearby = 1;
  SubsurfaceProcess proc2(Nearby, Phit, Nhit);
  tree->Lookup(Phit, proc2, md2);
  vector<CloseSubsurface> &subsurfacePts2 = proc2.subsurfacePts;
	

	float md=100*lu;
  Nearby = 400;
  SubsurfaceProcess proc(Nearby, subsurfacePts2[0].data->P, Nhit);  
  tree->Lookup(subsurfacePts2[0].data->P, proc, md);
  vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;

  
	if(debugflag)
		cout<<"md2 " << md2 << "    md :" << md<<endl;
	if(debugflag)
		cout<<" Zr " << Zr << "    Zv :" << Zv<<"   Fresnel: "<<Fdt<<endl;
	if(debugflag)
		cout<< subsurfacePts.size()<<endl;

  float r,dr,dv,c1,c2;
  for(u_int i=0; i<subsurfacePts.size(); i++)
   {
      r = Distance(subsurfacePts[i].data->P,subsurfacePts2[0].data->P);
			if(r==0)
				continue;
			//      r = Distance(subsurfacePts[i].data->P,Phit);
      dr = sqrt(r*r + Zr*Zr);
      dv = sqrt(r*r + Zv*Zv);
      c1 = Zr*(sigtr[1] + 1.0/dr);
      c2 = Zv*(sigtr[1] + 1.0/dv);
      
      c1 /= (dr*dr);
      c2 /= (dv*dv);
      
      c1 *= exp(-sigtr[1]*dr);
      c2 *= exp(-sigtr[1]*dv);
      float temp = Fdt*(c1 + c2)*(oneby4pi)*Ap;
			if(debugflag)
				cout<<"r: "<<r<<"    dr "<<dr<<"    dv " <<dv;
			if(debugflag)
				cout<<"    c1 "<<c1<<"    c2 " <<c2<<"    c1+c2: "<<(c1 + c2)<<endl;
			if(debugflag)
				cout<<"   temp: " << temp<<"   dE "<<subsurfacePts[i].data->dE<<endl;
      L += temp*subsurfacePts[i].data->dE;
    }
  
	//	cout << L << "   " <<endl;
  return L*Nearby/subsurfacePts.size();//(Fdt*L)/(Fdr*M_PI);
}
 
BlobSampler::BlobSampler(Blob* blob)
{
  tree = NULL;
  nLookup = 1;
  no_of_samples =0 ;
  
  Nsphere = 12000;

  mindistFactor = 0.02/1.5;//for sphere
  //mindistFactor = 0.0033/1.5;//for cylinder 
  
  lu = 0.02;//mindistFactor;
  
  averageLU = 0.00328112;
	  Ap = M_PI*(averageLU*averageLU)/4;
  cout<<"False Ap :  "<<Ap<<endl;
  
  //FindSamplesKDTREE(blob);
  //No lower PART??
  FindSamples(blob);
  //readSamples("scenes/sphere-150000.data");
  FindAverage();
  
	Ap=4*M_PI*(0.4*0.4)/totalSamplePoints.size();
  cout<<"Ap :  "<<Ap<<endl;
  
  /*Translucency PART*/
  float scale = 10; //this is because 10cm = 1 unit 
	//  float scale = 100; //this is because 10cm = 1 unit 
	/*
  siga[0] = 0.0021*scale;
  siga[1] = 0.0041*scale;
  siga[2] = 0.0071*scale;
  
  sigs[0] = 2.19*scale;
  sigs[1] = 2.62*scale;
  sigs[2] = 3.00*scale;
  */
  siga[0] = 0.0021*scale;
  siga[1] = 0.0041*scale;
  siga[2] = 0.0071*scale;
  
  sigs[0] = 2.19*scale;
  sigs[1] = 2.62*scale;
  sigs[2] = 3.00*scale;


  for(int i=0; i<3; i++)
    sigt[i] = sigs[i] + siga[i];
  
  for(int i=0; i<3; i++)
    sigtr[i] = sqrt(3*siga[i]*sigt[i]);
  
  float eta = 1.5;

  Fdr = -1.440/(eta*eta) + 0.710/eta + 0.668 + 0.0636*eta;//0.59673;
  cout<<"FDR    : "<<Fdr<<endl;;

  Fdt = 1 - Fdr;
  float A = (1+Fdr)/(1-Fdr);
  
  Zr = 0.0033;  //lu
  Zv = Zr*(1+(4.0/(3*A)));


	Zr*=20;
	Zv*=20;
}

void BlobSampler::readSamples(char *filename)
{
  FILE* file;
  file = fopen(filename,"r");
  
  //save the samples in the file
  fscanf(file,"%d\n", &no_of_samples);
  
  SubsurfaceData sd;
  totalSamplePoints.reserve(no_of_samples);

  for(int i=0; i<no_of_samples;i++)
    {
      fscanf(file,"%f %f %f %f %f %f\n",&sd.P.x,&sd.P.y, &sd.P.z,&sd.N.x,&sd.N.y,&sd.N.z);
      totalSamplePoints.push_back(sd);
    }
  fclose(file);

  //make the kdtree
  tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(totalSamplePoints);  
	cout << totalSamplePoints.size()<<endl;
}

BlobSampler::~BlobSampler()
{
  if(tree != NULL)
    delete tree;
}

void BlobSampler::FindAverage()
{
  averageLU = 0;
  float dev = 0;
  float max_dev = 0;
  
  //cout<<"Total Number of Samples = "<<no_of_samples<<endl;

  int good_samples = 0;
  
  for(int i=0; i<no_of_samples;i++){
    float mindist = INFINITY;
    
    SubsurfaceProcess proc(10, totalSamplePoints[i].P, totalSamplePoints[i].N);
    float md =  5*lu;
    tree->Lookup(totalSamplePoints[i].P, proc, md);
    vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;
    
    for(unsigned int u =0;  u < subsurfacePts.size(); u++){
      float dist = Distance(subsurfacePts[u].data->P,totalSamplePoints[i].P); 
      if( dist > 0.0000f && dist <mindist){
	mindist = dist;
      }
    }
    
    if(mindist < 10000){
      averageLU += mindist;
      good_samples++;
    }
  }
  
  averageLU /= good_samples;
 
  for(int i=0; i<no_of_samples;i++)
    {
      float mindist = INFINITY;
      
      SubsurfaceProcess proc(10, totalSamplePoints[i].P, totalSamplePoints[i].N);
      float md =  10*lu;
      tree->Lookup(totalSamplePoints[i].P, proc, md);
      vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;
      for(unsigned int u =0;  u < subsurfacePts.size(); u++)
	{
	  float dist = Distance(subsurfacePts[u].data->P,totalSamplePoints[i].P); 
	  if( dist > 0.0000f && dist <mindist)
	    mindist = dist;
	}
      
      if(mindist < 1000000)
	{
	  if(fabs(mindist -averageLU)>max_dev)
	    max_dev = fabs(mindist -averageLU);
	  
	  dev += (mindist -averageLU)*(mindist -averageLU);
	}
    }
 
  dev /= good_samples;//no_of_samples;
  
  cout<<"Good samples = " <<good_samples<<endl;
  cout<<"averageLU = "<<(averageLU)<<endl;
  cout<<"lu:  "<<lu<<endl;
  
  cout<<"std::dev = "<<sqrt(dev)<<endl;
  cout<<"max::dev = "<<max_dev<<endl;
  //exit(1);
}


void BlobSampler::FindSamples(Blob* blob)
{
  std::vector<std::vector<SubsurfaceData> > samplePoints;
  
  std::vector<SubsurfaceData> elementsVec;
  elementsVec.reserve(blob->elements.size());
  
  float* uniformsamples = new float[Nsphere*2];

  for(unsigned int i = 0; i<blob->elements.size();i++)  {
    SubsurfaceData dt;
    dt.P = blob->elements[i]->center;
    dt.N = Normal(0,1,0);
    dt.dE.c[0] = float(i);
    elementsVec.push_back(dt);
  }
  
  KdTree<SubsurfaceData,SubsurfaceProcess>* elementsTree = new KdTree<SubsurfaceData,SubsurfaceProcess>(elementsVec);
  
  for(unsigned int u=0; u < blob->elements.size(); u++) {
    float sphere_distance = 2*blob->elements[u]->rad;
    
    std::vector<int> closeSamples;
    std::vector<SubsurfaceData> localSamples;
    Point center = blob->elements[u]->center;
    
    /* Find the neighbours..kdtree is used for making thia part fast*/
    SubsurfaceProcess proc(15, center, Normal(0,1,0));
    elementsTree->Lookup(center, proc, sphere_distance);
    vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;
    
    for(u_int i =0;  i < subsurfacePts.size(); i++){
      u_int t = (int)(subsurfacePts[i].data->dE.c[0]);
      if(t < samplePoints.size())
				closeSamples.push_back(t);
    }
    
    /*
      for(unsigned int i=0; i<samplePoints.size(); i++)
      {
      if(Distance(center,blob->elements[i]->center) < sphere_distance)
      closeSamples.push_back(i);
      }
    */
    

		//    LatinHypercube(uniformsamples,Nsphere,2);

    for(unsigned int i=0; i<Nsphere; i++){
      Ray ray;
      float u1 = RandomFloat(0,1);
      float u2 = RandomFloat(0,1);
      
      Vector dir =  UniformSampleSphere(u1,u2);// - (Vector)center;
			//      Vector dir =  UniformSampleSphere(uniformsamples[2*i],uniformsamples[2*i+1]);//u1,u2);// - (Vector)center;

      ray.O = center;
      ray.D = dir;
      
      ray.mint = 0;
      ray.maxt = INFINITY;
      Point Phit;
      Normal Nhit;
      
      if(blob->IntersectBlob(ray, Phit, Nhit)) {
				float ilen = 1.0/Nhit.Length();
				Nhit = Nhit*ilen;
				
				float dist;
				Vector V1;
				Point PP = scalePointInv(Phit,blob->elements[u]->scaleTrans);
				V1 = PP - center;
				dist = Dot(V1, V1);
				
				if(dist <= blob->elements[u]->rad2){
					bool isTooclose = false;
					
					for(unsigned int m=0; m<closeSamples.size() && isTooclose == false;m++){
						for(unsigned int n=0; n<samplePoints[closeSamples[m]].size() && isTooclose == false;n++)
							if(Distance(samplePoints[closeSamples[m]][n].P, Phit) < lu){
								isTooclose = true;
								break;
							}
						if(isTooclose) break;
					}
					
					if(isTooclose != true){
						for(unsigned int m=0; m<localSamples.size(); m++){
							if(Distance(localSamples[m].P, Phit) < lu
								 /*&& Dot(localSamples[m].N,Nhit) > 0.80*/){
								//cout<<"TRUE"<<endl;
								isTooclose = true;
								break;
							}
						}
					}
					
					//check inside /outside
					/*Normal temp = Normal(Phit.x, Phit.y, Phit.z);
						if(isTooclose == false && Dot((temp.Hat()),Nhit) > 0)
						isTooclose = true;
					*/
					
					if(isTooclose == false){
						SubsurfaceData data;
						data.P = Phit;
						data.N = Nhit;
						localSamples.push_back(data);
						if(i%1000 == 0)
							cout<<i<<endl;
					}
				}
      }
    }
    
    samplePoints.push_back(localSamples);
  }
  
  no_of_samples =0;
  for(unsigned int i=0; i<samplePoints.size(); i++)
    no_of_samples += samplePoints[i].size();
  
  cout<<"total no of samples: "<<no_of_samples<<endl;
  
  
  totalSamplePoints.clear();
  totalSamplePoints.reserve(no_of_samples);
  
  for(unsigned int i=0; i<samplePoints.size(); i++)
    {
      for(unsigned int j=0; j<samplePoints[i].size(); j++)
	{
	  totalSamplePoints.push_back(samplePoints[i][j]);
	}
    }

  //make the kdtree
  tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(totalSamplePoints);
  

  //save the samples in the file
  FILE* file;
  file = fopen("samples.data","w");
  fprintf(file,"%d\n", no_of_samples);
  for(int i=0; i<no_of_samples;i++)
    {
      fprintf(file,"%f %f %f %f %f %f\n", totalSamplePoints[i].P.x,totalSamplePoints[i].P.y,totalSamplePoints[i].P.z,totalSamplePoints[i].N.x, totalSamplePoints[i].N.y, totalSamplePoints[i].N.z);
    }
  fclose(file);

  delete elementsTree;
  delete[] uniformsamples;
}


void BlobSampler::FindSamplesKDTREE(Blob* blob)
{
  unsigned int No_localPoints = 50;
  totalSamplePoints.reserve(Nsphere*blob->elements.size());
  std::vector<SubsurfaceData> localSamples;
  
  //  lu = blob->elements[0]->rad*mindistFactor;
  float luPlus = lu*1.001;

  for(unsigned int u=0; u < blob->elements.size(); u++)
    {
      Point center = blob->elements[u]->center;
      
      for(unsigned int i=0; i<Nsphere; i++)
	{
	  Ray ray;
	  float u1 = RandomFloat(0,1);
	  float u2 = RandomFloat(0,1);
	  
	  Vector dir =  UniformSampleSphere(u1,u2);// - (Vector)center;
	  //dir = HAT(dir);
	  
	  ray.O = center;
	  ray.D = dir;
	  
	  ray.mint = 0;
	  ray.maxt = INFINITY;
	  Point Phit;
	  Normal Nhit;
	  
	  if(blob->IntersectBlob(ray, Phit, Nhit))
	    {
	      float ilen = 1.0/Nhit.Length();
	      Nhit = Nhit*ilen;
	      
	      bool isTooclose = false;
	      if(tree!= NULL)
		{	
		  SubsurfaceProcess proc(1, Phit, Nhit);
		  tree->Lookup(Phit, proc, luPlus);
		  vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;
		  if(subsurfacePts.size()>0)
		    {
		      isTooclose = true;
		      cout<<"TREE"<<endl;
		    }
		  //cout<<"TREE-2"<<endl;
		}
	      
	      if(isTooclose == false)
		{
		  for(unsigned int m=0; m<localSamples.size(); m++)
		    {
		      if(Distance(localSamples[m].P, Phit) < lu
			 /*&& Dot(localSamples[m].N,Nhit) > 0.80*/)
			{
			  isTooclose = true;
			}
		    }
		}
	      
	      if(isTooclose == false)
		{
		  SubsurfaceData data;
		  data.P = Phit;
		  data.N = Nhit;
		  localSamples.push_back(data);
		}
	    }
	  
	  if(localSamples.size() > No_localPoints)
	    {
	      totalSamplePoints.insert(totalSamplePoints.end(), localSamples.begin(),localSamples.end());
	      localSamples.clear();
	      
	      if(tree != NULL)
		delete tree;
	      tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(totalSamplePoints);
	    }
	}
    }
  

  no_of_samples = totalSamplePoints.size();  
  //make the kdtree
  if(tree != NULL)
    delete tree;
  tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(totalSamplePoints);
  

  //save the samples in the file
  FILE* file;
  file = fopen("samples.data","w");
  fprintf(file,"%d\n", no_of_samples);
  for(int i=0; i<no_of_samples;i++)
    {
      fprintf(file,"%f %f %f %f %f %f\n", totalSamplePoints[i].P.x,totalSamplePoints[i].P.y,totalSamplePoints[i].P.z,totalSamplePoints[i].N.x, totalSamplePoints[i].N.y, totalSamplePoints[i].N.z);
    }
  fclose(file);
}


SubsurfaceProcess::SubsurfaceProcess(u_int mp, const Point &p,
			     const Normal &n)
  : P(p), N(n) {
  maxSubsurfaces = mp;
}

void SubsurfaceProcess::operator()(const SubsurfaceData &photon,
			       Float &maxDist) const {
	//  if (Dot(photon.N, N) < .707f) return;
  Float d2 = DistanceSquared(photon.P, P);
  if (subsurfacePts.size() == 0)
    subsurfacePts.reserve(maxSubsurfaces);
  if (subsurfacePts.size() < maxSubsurfaces) {
    // Add photon to unordered array of photons
    subsurfacePts.push_back(CloseSubsurface(&photon, d2));
    if (subsurfacePts.size() == maxSubsurfaces) {
      std::make_heap(subsurfacePts.begin(), subsurfacePts.end());
      maxDist = sqrtf(subsurfacePts[0].maxDist2);
    }
  }
  else {
    // Remove most distant photon from heap and add new photon
    std::pop_heap(subsurfacePts.begin(), subsurfacePts.end());
    subsurfacePts[maxSubsurfaces-1] = CloseSubsurface(&photon, d2);
    std::push_heap(subsurfacePts.begin(), subsurfacePts.end());
    maxDist = sqrtf(subsurfacePts[0].maxDist2);
  }
}

#endif


//void BlobSampler::sample1(Blob* blob)
/*{
  unsigned int Nsphere = 5;
  //  unsigned int max_trials = Nsphere;
  
  unsigned int NsamplesMax = 50;
  float invsqrN = 2.0/sqrt(Nsphere);
  
  std::vector<SubsurfaceData> samplePoints;
  
  std::vector<SubsurfaceData> totalSamplePoints;

*///FILE* file;
/* file = fopen("samples.data","w");
      
  samplePoints.clear();
  
  for(unsigned int u=0; u<blob->elements.size(); u++)
    {
      Point center = blob->elements[u]->center;
            
      for(unsigned int i=0; i<Nsphere; i++)
      {
	  Ray ray;
	  float u1 = RandomFloat(0,1);
	  float u2 = RandomFloat(0,1);

	  Vector dir = (Vector)center - UniformSampleSphere(u1,u2);
	  dir = HAT(dir);
	  
	  ray.O = center;
	  ray.D = dir;
	  
	  Point Phit;
	  Normal Nhit;
	  
	  if(blob->IntersectBlob(ray, Phit, Nhit) == true)
	    {
	      bool isTooclose = false;
	      float mindistref = blob->elements[u]->rad*invsqrN;
	      
	      if (tree)
		{
		  SubsurfaceProcess proc(nLookup, Phit, Nhit);
		  
		  tree->Lookup(Phit, proc, mindistref);
		  vector<CloseSubsurface> &subsurfacePts = proc.subsurfacePts;
		  if(subsurfacePts.size() > 0) 
		    {
		      if(Distance(subsurfacePts[0].data->P, Phit) < mindistref)
			{
			  isTooclose = true;
			}
		    }
		}	
	      
	      //cout<<i<<endl;

	      if(isTooclose == false)
		{
		  SubsurfaceData data;
		  data.P = Phit;
		  data.N = Nhit;
		  samplePoints.push_back(data);
		  //		  fprintf(file,"%f %f %f\n", Phit.x,Phit.y,Phit.z);
		}
	    }
	}

      //make the tree again
      if(samplePoints.size() > NsamplesMax)
	{
	  cout<<"start"<<endl;
	  for(unsigned int v=0; v<samplePoints.size();v++)
	    totalSamplePoints.push_back(samplePoints[v]);
	  if(tree != NULL)
	    delete tree;
	  tree = new KdTree<SubsurfaceData,SubsurfaceProcess>(totalSamplePoints);
	  samplePoints.clear();
	  cout<<"tree make  "<<u<<endl;
	}
    }

  cout<<"Total number of samples: "<<totalSamplePoints.size()<<endl;
  for(unsigned int i=0; i<totalSamplePoints.size();i++)
    fprintf(file,"%f %f %f\n", totalSamplePoints[i].P.x,totalSamplePoints[i].P.y,totalSamplePoints[i].P.z);
  fclose(file);
} 
*/
