// trianglemesh.h*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#ifndef TRIANGLEMESH_H
#define TRIANGLEMESH_H
#include "lrt.h"
#include "shapes.h"
#include "paramset.h"
// TriangleMesh Declarations
class TriangleMesh: public Shape {
public:
	// TriangleMesh Interface
	TriangleMesh(const Transform &o2w, int ntris, int nverts, const int *vptr,
		const Point *P, const Float *uv = NULL);
	~TriangleMesh();
	BBox ObjectBound() const;
	BBox WorldBound() const;
	bool CanIntersect() const { return false; }
	void Refine(vector<RefinedShape> &refined) const;
	friend class Triangle;
	template <class T> friend class VertexTexture;
protected:
	// TriangleMesh Data
	int ntris;
	int nverts;
	int *vertexIndex;
	Point *p;
	Float *uvs;
};
class Triangle : public Shape {
public:
	// Triangle Interface
	Triangle(const Transform &o2w, TriangleMesh *m, int n)
			: Shape(o2w) {
		mesh = m;
		v = &mesh->vertexIndex[3*n];
		// Update created triangles stats
		static StatsCounter trisMade("Geometry", "Triangles created");
		++trisMade;
	}
	BBox ObjectBound() const;
	BBox WorldBound() const;
	void GetUVs(Float uv[3][2]) const;
	bool Intersect(const Ray &ray, Float *thitp, DifferentialGeometry *dg) const;
	bool IntersectP(const Ray &ray) const;
	Float Area() const;
//private:
	// Triangle Data
	Reference<TriangleMesh> mesh;
	int *v;
};
#endif // TRIANGLEMESH_H
