// trunccone.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "lrt.h"
#include "shapes.h"
// Cone Declarations
class TruncCone: public Shape {
public:
	// Cone Interface
	TruncCone(const Transform &o2w, Float height, Float rad1, Float rad2);
	BBox ObjectBound() const;
	bool Intersect(const Ray &ray, Float *thitp, DifferentialGeometry *dg) const;
	bool IntersectP(const Ray &ray) const;
	Float Area() const;
protected:
	// Cone Data
	Float radius, radius2, height, truncheight, phiMax;
	//radius2 smaller...
};
// Cone Methods
TruncCone::TruncCone(const Transform &o2w, Float ht, Float rad1, Float rad2)
	: Shape(o2w) {
	radius = rad1;
	radius2 = rad2;
	truncheight = ht;
	phiMax=Radians(360.0f);
	if(radius2<radius)
		height=radius*ht/(radius-radius2);
	else height=10000000.0;
}
BBox TruncCone::ObjectBound() const {
	Point p1 = Point( -radius, -radius, 0 );
	Point p2 = Point(  radius,  radius, truncheight );
	return BBox( p1, p2 );
}
bool TruncCone::Intersect(const Ray &r, Float *thitp,
		DifferentialGeometry *dg) const {
	Float phi;
	Point Phit;
	// Transform [[Ray]] to object space
	Ray ray;
	WorldToObject(r, &ray);
	// Compute quadratic cone coefficients
	Float k = radius / height;
	k = k*k;
	Float A = ray.D.x * ray.D.x + ray.D.y * ray.D.y -
		k * ray.D.z * ray.D.z;
	Float B = 2 * (ray.D.x * ray.O.x + ray.D.y * ray.O.y -
		k * ray.D.z * (ray.O.z-height) );
	Float C = ray.O.x * ray.O.x + ray.O.y * ray.O.y -
		k * (ray.O.z -height) * (ray.O.z-height);
	
	// Solve quadratic equation for [[t]] values
	Float t0, t1;
	if (!Quadratic(A, B, C, &t0, &t1))
		return false;
	// Compute intersection distance along ray
	if (t0 > ray.maxt || t1 < ray.mint)
		return false;
	Float thit = t0;
	if (t0 < ray.mint) {
		thit = t1;
		if (thit > ray.maxt) return false;
	}
	// Compute cone inverse mapping
	Phit = ray(thit);
	phi = atan2f(Phit.y, Phit.x);
	if (phi < 0.) phi += 2.f*M_PI;
	// Test cone intersection against clipping parameters
	if (Phit.z < 0 || Phit.z > truncheight) {
		if (thit == t1) return false;
		thit = t1;
		if (t1 > ray.maxt) return false;
		// Compute cone inverse mapping
		Phit = ray(thit);
		phi = atan2f(Phit.y, Phit.x);
		if (phi < 0.) phi += 2.f*M_PI;
		if (Phit.z < 0 || Phit.z > truncheight)
			return false;
	}
	// Fill in [[DifferentialGeometry]] from cone hit
	Float u = phi / phiMax;
	Float v = Phit.z / truncheight;

/*
	// Compute cylinder \dPdu and \dPdv
	Vector dPdu(-phiMax * Phit.y, phiMax * Phit.x, 0);
	Vector dPdv(0, 0, zmax - zmin);
	// Compute cylinder \dNdu and \dNdv
	Vector d2Pduu = -phiMax * phiMax * Vector(Phit.x, Phit.y, 0);
	Vector d2Pduv(0, 0, 0), d2Pdvv(0, 0, 0);
	// Compute coefficients for fundamental forms
	Float E = Dot(dPdu, dPdu);
	Float F = Dot(dPdu, dPdv);
	Float G = Dot(dPdv, dPdv);
	Vector N = Cross(dPdu, dPdv);
	Float e = Dot(N, d2Pduu);
	Float f = Dot(N, d2Pduv);
	Float g = Dot(N, d2Pdvv);
	// Compute \dNdu and \dNdv from fundamental form coefficients
	Float invEGF2 = 1.f / (E*G - F*F);
	Vector dNdu = (f*F - e*G) * invEGF2 * dPdu +
		(e*F - f*E) * invEGF2 * dPdv;
	Vector dNdv = (g*F - f*G) * invEGF2 * dPdu +
		(f*F - g*E) * invEGF2 * dPdv;
	*dg = DifferentialGeometry(ObjectToWorld(Phit), ObjectToWorld(dPdu),
		ObjectToWorld(dPdv), ObjectToWorld(dNdu), ObjectToWorld(dNdv),
		u, v, this);
*/

	// Compute cone \dPdu and \dPdv
	Vector dPdu(-phiMax * Phit.y, phiMax * Phit.x, 0);
	Vector dPdv(0, 0, truncheight);
	// Compute cone \dNdu and \dNdv
	Vector d2Pduu = -phiMax * phiMax * Vector(Phit.x, Phit.y, 0.);
	Vector d2Pduv(0,0,0);
	Vector d2Pdvv(0, 0, 0);
	// Compute coefficients for fundamental forms
	Float E = Dot(dPdu, dPdu);
	Float F = Dot(dPdu, dPdv);
	Float G = Dot(dPdv, dPdv);
	Vector N = Cross(dPdu, dPdv);
	Float e = Dot(N, d2Pduu);
	Float f = Dot(N, d2Pduv);
	Float g = Dot(N, d2Pdvv);
	// Compute \dNdu and \dNdv from fundamental form coefficients
	Float invEGF2 = 1.f / (E*G - F*F);
	Vector dNdu = (f*F - e*G) * invEGF2 * dPdu +
		(e*F - f*E) * invEGF2 * dPdv;
	Vector dNdv = (g*F - f*G) * invEGF2 * dPdu +
		(f*F - g*E) * invEGF2 * dPdv;
	*dg = DifferentialGeometry(ObjectToWorld(Phit), ObjectToWorld(dPdu),
		ObjectToWorld(dPdv), ObjectToWorld(dNdu), ObjectToWorld(dNdv),
		u, v, this);
	// Update [[thitp]] for quadric intersection
	*thitp = thit;
	return true;
}

bool TruncCone::IntersectP(const Ray &r) const {
	Point Phit;
	// Transform [[Ray]] to object space
	Ray ray;
	WorldToObject(r, &ray);
	// Compute quadratic cone coefficients
	Float k = radius / height;
	k = k*k;
	Float A = ray.D.x * ray.D.x + ray.D.y * ray.D.y -
		k * ray.D.z * ray.D.z;
	Float B = 2 * (ray.D.x * ray.O.x + ray.D.y * ray.O.y -
		k * ray.D.z * (ray.O.z-height) );
	Float C = ray.O.x * ray.O.x + ray.O.y * ray.O.y -
		k * (ray.O.z -height) * (ray.O.z-height);
	
	// Solve quadratic equation for [[t]] values
	Float t0, t1;
	if (!Quadratic(A, B, C, &t0, &t1))
		return false;
	// Compute intersection distance along ray
	if (t0 > ray.maxt || t1 < ray.mint)
		return false;
	Float thit = t0;
	if (t0 < ray.mint) {
		thit = t1;
		if (thit > ray.maxt) return false;
	}
	// Compute cone inverse mapping
	Phit = ray(thit);
	// Test cone intersection against clipping parameters
	if (Phit.z < 0 || Phit.z > truncheight) {
		if (thit == t1) return false;
		thit = t1;
		if (t1 > ray.maxt) return false;
		// Compute cone inverse mapping
		Phit = ray(thit);
		if (Phit.z < 0 || Phit.z > truncheight)
			return false;
	}
	return true;
}
Float TruncCone::Area() const {
	return 0;
}

extern "C" Reference<Shape>CreateShape( const Transform &o2w, const ParamSet &params )
{
	Float radius = params.FindOneFloat( "_radius1", 1 );
	Float radius2 = params.FindOneFloat( "_radius2", 0.9 );
	Float height = params.FindOneFloat( "_height", 1 );
	return new TruncCone( o2w, height, radius, radius2 );
}
