// contrast.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "tonemap.h"
// ContrastOp Declarations
class ContrastOp : public ToneMap {
public:
	void Map(Float *xyz, int xRes, int yRes,
		const DisplayInfo &di) const;
};
// ContrastOp Method Definitions
void ContrastOp::Map(Float *xyz, int xRes, int yRes,
		const DisplayInfo &di) const {
	// Compute world adaptation luminance
	Float Ywa = 0.;
	for (int i = 0; i < xRes * yRes; ++i)
		if (Y(i) > 0) Ywa += logf(Y(i));
	Ywa = expf(Ywa / (xRes * yRes));
	// Compute contrast-preserving scalefactor
	Float scale = powf((1.219f + powf(di.displayAdaptationY, 0.4f)) /
		(1.219f + powf(Ywa, 0.4f)), 2.5f);
	// Apply [[scale]] to all image pixels
	for (int i = 0; i < xRes * yRes; ++i) {
		X(i) *= scale;
		Y(i) *= scale;
		Z(i) *= scale;
	}
}
extern "C" ToneMap *CreateToneMap(const ParamSet &ps) {
	return new ContrastOp;
}
