// nonlinear.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "tonemap.h"
// NonLinearOp Declarations
class NonLinearOp : public ToneMap {
	void Map(Float *xyz, int xRes, int yRes,
		const DisplayInfo &di) const;
};
// NonLinearOp Method Definitions
void NonLinearOp::Map(Float *xyz, int xRes, int yRes,
		const DisplayInfo &di) const {
	// Compute world adaptation luminance
	Float Ywa = 0.;
	for (int i = 0; i < xRes * yRes; ++i)
		if (Y(i) > 0) Ywa += logf(Y(i));
	Ywa = expf(Ywa / (xRes * yRes));
	Float invY2 = 1.f / (Ywa * Ywa);
	for (int i = 0; i < xRes * yRes; ++i) {
		Float scale = di.maxDisplayY *
			(1.f + Y(i) * invY2) / (1.f + Y(i));
		X(i) *= scale;
		Y(i) *= scale;
		Z(i) *= scale;
	}
}
extern "C" ToneMap *CreateToneMap(const ParamSet &ps) {
	return new NonLinearOp;
}
