// exponential.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "volume.h"
// ExponentialMist Declarations
class ExponentialMist : public DensityRegion {
public:
	// ExponentialMist Methods
	ExponentialMist(const Spectrum &sa, const Spectrum &ss, Float gg,
		 	const Spectrum &emit, const BBox &e, const Transform &v2w,
			Float a, Float b)
		: DensityRegion(sa, ss, gg, emit, v2w) {
		extent = e;
		A = a;
		B = b;
	}
	BBox WorldBound() const { return WorldToVolume.GetInverse()(extent); }
	bool Intersect(const Ray &r, Float *t0, Float *t1) const {
		Ray ray = WorldToVolume(r);
		return extent.IntersectP(ray, t0, t1);
	}
	Float density(const Point &Pobj) const {
		return A * exp(-B * Pobj.z);
	}
	Spectrum tau(const Ray &r) const {
		Ray ray = WorldToVolume(r);
		Float t0, t1;
		if (!extent.IntersectP(ray, &t0, &t1)) return 0.;
		Float dist = Distance(ray(t0), ray(t1));
		return (A * dist)/(B * ray.D.z) *
			(exp(-ray(t1).z) - exp(-ray(t0).z)) *
			(sig_a + sig_s);
	}
private:
	// ExponentialMist Private Data
	BBox extent;
	Float A, B;
};
// ExponentialMist Definitions
extern "C" VolumeRegion *CreateVolumeRegion(const Transform &volume2world,
		const ParamSet &params) {
	Spectrum sigma_a = params.FindOneSpectrum("sigma_a", 0.);
	Spectrum sigma_s = params.FindOneSpectrum("sigma_s", 0.);
	Float g = params.FindOneFloat("g", 0.);
	Spectrum Le = params.FindOneSpectrum("Le", 0.);
	Point p0 = params.FindOnePoint("p0", Point(0,0,0));
	Point p1 = params.FindOnePoint("p1", Point(1,1,1));
	Float A = params.FindOneFloat("A", 1.);
	Float B = params.FindOneFloat("B", 1.);
	return new ExponentialMist(sigma_a, sigma_s, g, Le, BBox(p0, p1),
		volume2world, A, B);
}
