// homogeneous.cc*
// Source Code Copyright
/*
 * lrt source code Copyright(c) 1998-2003 Matt Pharr and Greg Humphreys
 *
 * All Rights Reserved.
 * For non-commercial use only.
 * No warranty, express or implied, for this software.
 */
#include "volume.h"
// HomogeneousRegion Declarations
class HomogeneousRegion : public VolumeRegion {
public:
	// HomogeneousRegion Methods
	HomogeneousRegion(const Spectrum &sa, const Spectrum &ss, Float gg,
		 	const Spectrum &emit, const BBox &e,
			const Transform &v2w) {
		WorldToVolume = v2w.GetInverse();
		sig_a = sa;
		sig_s = ss;
		g = gg;
		le = emit;
		extent = e;
	}
	BBox WorldBound() const { return WorldToVolume.GetInverse()(extent); }
	bool Intersect(const Ray &r, Float *t0, Float *t1) const {
		Ray ray = WorldToVolume(r);
		return extent.IntersectP(ray, t0, t1);
	}
	Spectrum sigma_a(const Point &p, const Vector &) const {
		return extent.Inside(WorldToVolume(p)) ? sig_a : 0.;
	}
	Spectrum sigma_s(const Point &p, const Vector &) const {
		return extent.Inside(WorldToVolume(p)) ? sig_s : 0.;
	}
	Spectrum sigma_t(const Point &p, const Vector &) const {
		return extent.Inside(WorldToVolume(p)) ? (sig_a + sig_s) : 0.;
	}
	Spectrum Le(const Point &p, const Vector &) const {
		return extent.Inside(WorldToVolume(p)) ? le : 0.;
	}
	Float phase(const Point &p, const Vector &wi, const Vector &wo) const {
		if (!extent.Inside(WorldToVolume(p))) return 0.;
		return PhaseHG(wi, wo, g);
	}
	Spectrum tau(const Ray &r) const {
		Ray ray = WorldToVolume(r);
		Float t0, t1;
		if (!extent.IntersectP(ray, &t0, &t1)) return 0.;
		return Distance(ray(t0), ray(t1)) * (sig_a + sig_s);
	}
private:
	// HomogeneousRegion Private Data
	Spectrum sig_a, sig_s, le;
	Float g;
	BBox extent;
	Transform WorldToVolume;
};
// HomogeneousRegion Definitions
extern "C" VolumeRegion *CreateVolumeRegion(const Transform &volume2world,
		const ParamSet &params) {
	Spectrum sigma_a = params.FindOneSpectrum("sigma_a", 0.);
	Spectrum sigma_s = params.FindOneSpectrum("sigma_s", 0.);
	Float g = params.FindOneFloat("g", 0.);
	Spectrum Le = params.FindOneSpectrum("Le", 0.);
	Point p0 = params.FindOnePoint("p0", Point(0,0,0));
	Point p1 = params.FindOnePoint("p1", Point(1,1,1));
	return new HomogeneousRegion(sigma_a, sigma_s, g, Le, BBox(p0, p1),
		volume2world);
}
